@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
  
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,9 +32,8 @@
 #include <syslog.h>
 #include <errno.h>
 
-#include <radius.h> 
-#include <radpaths.h> 
-#include <radutmp.h>
+#include <common.h> 
+#include <radius/radutmp.h>
 
 /* indicate the following groups are defined */
 #define PAM_SM_AUTH
@@ -86,12 +87,12 @@ _cleanup_string(pam_handle_t *pamh, void *x, int error_status)
 
 /* Clean up the temporary A/V pair list */
 static void
-_cleanup_fake_pairlist(VALUE_PAIR *p)
+_cleanup_fake_pairlist(grad_avp_t *p)
 {
-	VALUE_PAIR *next;
+	grad_avp_t *next;
 	while (p) {
 		next = p->next;
-		efree(p);
+		grad_free(p);
 		p = next;
 	}
 }
@@ -112,44 +113,21 @@ _pam_vlog(int err, const char *format, va_list args)
 }
 
 static void
-_pam_log 
-#if STDC_HEADERS
-        (int err, const char *format, ...)
-#else
-        (err, format, va_alist)
-        int err;
-        const char *format;
-        va_dcl
-#endif
+_pam_log(int err, const char *format, ...)
 {
         va_list args;
 
-#if STDC_HEADERS
         va_start(args, format);
-#else
-	va_start(args);
-#endif
         _pam_vlog(err, format, args);
         va_end(args);
 }
 
 static void
-_pam_debug
-#if STDC_HEADERS
-          (char *format, ...)
-#else
-          (format, va_alist)
-        char *format;
-        va_dcl
-#endif
+_pam_debug(char *format, ...)
 {
         va_list args;
 
-#if STDC_HEADERS
 	va_start(args, format);
-#else
-        va_start(args);
-#endif
         _pam_vlog(LOG_DEBUG, format, args);
         va_end(args);
 }
@@ -170,7 +148,8 @@ int priority[] = {
 
 void
 vlog(int level,
-     const char *file, int line,
+     const grad_request_t *req,
+     const grad_locus_t *loc,
      const char *func_name, int en,
      const char *fmt, va_list ap)
 {
@@ -234,14 +213,14 @@ _pam_parse(pam_handle_t *pamh, int argc, const char **argv)
 {
         int ctrl=0;
 	int in_attr = 0;
-	VALUE_PAIR *head = NULL, *tail = NULL;
+	grad_avp_t *head = NULL, *tail = NULL;
 	
         /* step through arguments */
         for (ctrl=0; argc-- > 0; ++argv) {
 		
 		if (in_attr) {
 			char *p = strchr(*argv, '=');
-			VALUE_PAIR *pair;
+			grad_avp_t *pair;
 			
 			if (!p) {
 				_pam_log(LOG_ERR,
@@ -250,7 +229,7 @@ _pam_parse(pam_handle_t *pamh, int argc, const char **argv)
 			}
 			*p++ = 0;
 			
-			pair = emalloc(sizeof(*pair));
+			pair = grad_emalloc(sizeof(*pair));
 			pair->next = NULL;
 			pair->name = (char *) *argv;
 			pair->avp_strvalue = p;
@@ -311,28 +290,28 @@ _pam_parse(pam_handle_t *pamh, int argc, const char **argv)
 static void
 _cleanup_server_queue(pam_handle_t *pamh, void *x, int error_status)
 {
-        rad_clt_destroy_queue(x);
+        grad_client_destroy_queue(x);
 }
 
 static void
 _cleanup_request(pam_handle_t *pamh, void *x, int error_status)
 {
-        radreq_free((RADIUS_REQ*)x);
+        grad_request_free((grad_request_t*)x);
 }
 
 int
 _read_client_config(pam_handle_t *pamh)
 {
         int errcnt = 0;
-        RADIUS_SERVER_QUEUE *queue;
+        grad_server_queue_t *queue;
         
-        queue = rad_clt_create_queue(1, 0, 0);
+        queue = grad_client_create_queue(1, 0, 0);
         if (!queue)
                 return -1;
         /*
          * Consistency check
          */
-        if (list_count(queue->servers) == 0) {
+        if (grad_list_count(queue->servers) == 0) {
                 _pam_log(LOG_ERR, "config: no server selected");
                 errcnt++;
         }
@@ -344,7 +323,7 @@ _read_client_config(pam_handle_t *pamh)
 
         if (errcnt) {
                 /* free allocated memory */
-                rad_clt_destroy_queue(queue);
+                grad_client_destroy_queue(queue);
         } else {
                 errcnt = pam_set_data(pamh,
                                       "radius_server_queue", 
@@ -355,7 +334,7 @@ _read_client_config(pam_handle_t *pamh)
                                  "can't keep data [%s]: %s",
                                  "radius_server_queue",
                                  pam_strerror(pamh, errcnt));
-                        rad_clt_destroy_queue(queue);
+                        grad_client_destroy_queue(queue);
                         errcnt = 1;
                 }
         }
@@ -370,9 +349,9 @@ _pam_init_radius_client(pam_handle_t *pamh)
         DEBUG(100,("enter _pam_init_radius_client"));
 
         radius_dir = radius_confdir;
-        radpath_init();
-        if (dict_init()) {
-                _pam_log(LOG_CRIT, "dict_init failed");
+        grad_path_init();
+        if (grad_dict_init()) {
+                _pam_log(LOG_CRIT, "grad_dict_init failed");
                 return 1;
         }
         rc = _read_client_config(pamh);
@@ -385,12 +364,13 @@ _pam_init_radius_client(pam_handle_t *pamh)
 static int
 _radius_auth(pam_handle_t *pamh, char *name, char *password)
 {
-        RADIUS_SERVER_QUEUE *queue;
+        grad_server_queue_t *queue;
         int retval;
-        VALUE_PAIR *pairs, *namepair, *add_pair = NULL;
-        RADIUS_REQ *authreq;
-        DICT_VALUE *dv;
-        
+        grad_avp_t *pairs, *namepair, *add_pair = NULL;
+        grad_request_t *authreq;
+        grad_dict_value_t *dv;
+	grad_locus_t loc = { __FILE__, __LINE__ }; /* FIXME */
+
         retval = pam_get_data(pamh,
                               "radius_server_queue", 
                               (const void **)&queue);
@@ -413,50 +393,53 @@ _radius_auth(pam_handle_t *pamh, char *name, char *password)
          * Create authentication request
          */
         pairs = NULL;
-        avl_add_pair(&pairs,
-		     namepair = avp_create_string(DA_USER_NAME, name));
-        avl_add_pair(&pairs, avp_create_string(DA_USER_PASSWORD, password));
-        avl_add_pair(&pairs, avp_create_integer(DA_NAS_IP_ADDRESS,
-					        queue->source_ip));
+	namepair = grad_avp_create_string(DA_USER_NAME, name);
+        grad_avl_add_pair(&pairs, namepair);
+        grad_avl_add_pair(&pairs,
+			  grad_avp_create_string(DA_USER_PASSWORD, password));
+        grad_avl_add_pair(&pairs,
+			  grad_avp_create_integer(DA_NAS_IP_ADDRESS,
+						  queue->source_ip));
 	/* Add any additional attributes */
 	for (; add_pair; add_pair = add_pair->next) {
-		VALUE_PAIR *p = install_pair(__FILE__, __LINE__,
-                                             add_pair->name, 
-                                             OPERATOR_EQUAL,
-				             add_pair->avp_strvalue);
-		avl_add_pair(&pairs, p);
+		grad_avp_t *p = grad_create_pair(&loc,
+						 add_pair->name, 
+						 grad_operator_equal,
+						 add_pair->avp_strvalue);
+		grad_avl_add_pair(&pairs, p);
 	}
 	/* For compatibility with previous versions handle service_type */
         if (service_type &&
-            (dv = value_name_to_value(service_type, DA_SERVICE_TYPE))) {
+            (dv = grad_value_name_to_value(service_type, DA_SERVICE_TYPE))) {
                 DEBUG(10, ("adding Service-Type=%d", dv->value));
-                avl_add_pair(&pairs,
-			     avp_create_integer(DA_SERVICE_TYPE, dv->value));
+                grad_avl_add_pair(&pairs,
+				  grad_avp_create_integer(DA_SERVICE_TYPE,
+							  dv->value));
         }
-        authreq = rad_clt_send(queue,
-			       PORT_AUTH, RT_AUTHENTICATION_REQUEST, pairs);
+        authreq = grad_client_send(queue,
+				   GRAD_PORT_AUTH, RT_ACCESS_REQUEST, pairs);
         if (authreq == NULL) {
                 _pam_log(LOG_ERR,
                          "no response from radius server");
-                avl_free(pairs);
+                grad_avl_free(pairs);
                 return PAM_AUTHINFO_UNAVAIL;
         }
 
         switch (authreq->code) {
-        case RT_AUTHENTICATION_ACK:
+        case RT_ACCESS_ACCEPT:
                 break;
 		
-        case RT_AUTHENTICATION_REJECT:
+        case RT_ACCESS_REJECT:
                 /* FIXME: radius may have returned Reply-Message attribute.
                  * we should return it to the caller
                  */
-                radreq_free(authreq);
+                grad_request_free(authreq);
                 return PAM_USER_UNKNOWN;
 		
         default:
                 _pam_log(LOG_CRIT,
                          "received unexpected response: %d", authreq->code);
-                radreq_free(authreq);
+                grad_request_free(authreq);
                 return PAM_AUTH_ERR;
         }
 
@@ -471,16 +454,16 @@ _radius_auth(pam_handle_t *pamh, char *name, char *password)
                 _pam_log(LOG_CRIT, 
                          "can't keep data authreq: %s",
                          pam_strerror(pamh, retval));
-                radreq_free(authreq);
+                grad_request_free(authreq);
         }
         /* add username to the response (do we need it still?) */
-        avl_add_pair(&authreq->request, avp_dup(namepair));
+        grad_avl_add_pair(&authreq->avlist, grad_avp_dup(namepair));
 
         return PAM_SUCCESS;
 }
 
 static void
-add_stat_pairs(VALUE_PAIR *plist)
+add_stat_pairs(grad_avp_t *plist)
 {
 	/* FIXME: noop */
 }
@@ -488,12 +471,13 @@ add_stat_pairs(VALUE_PAIR *plist)
 static int
 _radius_acct(pam_handle_t *pamh, struct radutmp *ut)
 {
-        RADIUS_SERVER_QUEUE *queue;
+        grad_server_queue_t *queue;
         int retval;
-        VALUE_PAIR *pairs, *add_pair = NULL;
-        RADIUS_REQ *req;
-        DICT_VALUE *dv;
+        grad_avp_t *pairs, *add_pair = NULL;
+        grad_request_t *req;
+        grad_dict_value_t *dv;
         int type;
+	grad_locus_t loc = { __FILE__, __LINE__ }; /* FIXME */
 	
         retval = pam_get_data(pamh,
                               "radius_server_queue", 
@@ -518,20 +502,25 @@ _radius_acct(pam_handle_t *pamh, struct radutmp *ut)
          * Create accounting request
          */
         pairs = NULL;
-        avl_add_pair(&pairs, avp_create_string(DA_USER_NAME, ut->login));
+        grad_avl_add_pair(&pairs,
+			  grad_avp_create_string(DA_USER_NAME, ut->login));
 
-	avl_add_pair(&pairs, avp_create_integer(DA_NAS_IP_ADDRESS,
-					        queue->source_ip));
-	avl_add_pair(&pairs, avp_create_integer(DA_NAS_PORT_ID, ut->nas_port));
+	grad_avl_add_pair(&pairs,
+			  grad_avp_create_integer(DA_NAS_IP_ADDRESS,
+						  queue->source_ip));
+	grad_avl_add_pair(&pairs,
+			  grad_avp_create_integer(DA_NAS_PORT_ID,
+						  ut->nas_port));
 	
-	avl_add_pair(&pairs,
-		     avp_create_string(DA_ACCT_SESSION_ID, ut->session_id));
+	grad_avl_add_pair(&pairs,
+			  grad_avp_create_string(DA_ACCT_SESSION_ID,
+						 ut->session_id));
 	
 	/* Add any additional attributes */
 	for (; add_pair; add_pair = add_pair->next) {
-		VALUE_PAIR *p = install_pair(__FILE__, __LINE__,
+		grad_avp_t *p = grad_create_pair(&loc,
                                              add_pair->name, 
-                                             OPERATOR_EQUAL,
+                                             grad_operator_equal,
 				             add_pair->avp_strvalue);
 		switch (p->attribute) {
                 case DA_LOGIN_IP_HOST:
@@ -551,7 +540,7 @@ _radius_acct(pam_handle_t *pamh, struct radutmp *ut)
 			ut->porttype = p->avp_lvalue;
                         break;
 		}
-		avl_add_pair(&pairs, p);
+		grad_avl_add_pair(&pairs, p);
 	}
 
 	switch (ut->type) {
@@ -563,19 +552,22 @@ _radius_acct(pam_handle_t *pamh, struct radutmp *ut)
 	case P_IDLE:
 		type = DV_ACCT_STATUS_TYPE_STOP;
 		radutmp_putent(radutmp_path, ut, type);
-		avl_add_pair(&pairs,
-			     avp_create_integer(DA_ACCT_SESSION_TIME,
-					        ut->duration));
+		grad_avl_add_pair(&pairs,
+				  grad_avp_create_integer(DA_ACCT_SESSION_TIME,
+							  ut->duration));
 		add_stat_pairs(pairs);
 	}
 					
-	avl_add_pair(&pairs, avp_create_integer(DA_ACCT_STATUS_TYPE, type));
+	grad_avl_add_pair(&pairs,
+			  grad_avp_create_integer(DA_ACCT_STATUS_TYPE,
+						  type));
 
-	req = rad_clt_send(queue, PORT_ACCT, RT_ACCOUNTING_REQUEST, pairs);
+	req = grad_client_send(queue, GRAD_PORT_ACCT, RT_ACCOUNTING_REQUEST, 
+                               pairs);
         if (req == NULL) {
                 _pam_log(LOG_ERR,
                          "no response from radius server");
-                avl_free(pairs);
+                grad_avl_free(pairs);
                 return PAM_SESSION_ERR;
         }
 
@@ -587,7 +579,7 @@ _radius_acct(pam_handle_t *pamh, struct radutmp *ut)
 		retval = PAM_SESSION_ERR;
 	}
 	
-	radreq_free(req);
+	grad_request_free(req);
         return retval;
 }
 
