@@ -17,11 +17,14 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define SQLT_NONE     0
-#define SQLT_MYSQL    1
-#define SQLT_POSTGRES 2
-#define SQLT_ODBC     3
-#define SQLT_MAX      4
+#ifdef HAVE_LIBLTDL
+# define DECL_SQL_DISPATCH_TAB(mod) \
+  SQL_DISPATCH_TAB RDL_EXPORT(mod,dispatch_tab)
+#else
+# define __s_cat2__(a,b) a ## b 
+# define DECL_SQL_DISPATCH_TAB(mod) \
+  SQL_DISPATCH_TAB __s_cat2__(mod,_dispatch_tab)
+#endif
 
 #ifdef USE_SQL
 
@@ -32,27 +35,6 @@
 # define SQL_CACHE_SIZE 16
 typedef char **SQL_TUPLE;
 
-typedef struct {
-	char *query;
-	size_t ntuples;
-	size_t nfields;
-	SQL_TUPLE *tuple;
-} SQL_RESULT;
-
-struct sql_connection {
-	int    interface;        /* One of SQLT_ values */
-        int    type;             /* One of SQL_ values */
-        int    connected;        /* Connected to the database? */
-        int    destroy_on_close; /* Should the connection be closed upon
-				    the end of a transaction */
-        time_t last_used;        /* Time it was lastly used */
-        void   *data;            /* connection-specific data */
-	
-	SQL_RESULT *cache[SQL_CACHE_SIZE];
-	size_t head;
-	size_t tail;
-};
-
 enum radius_sql_query {
 	auth_query,
 	group_query,
@@ -85,25 +67,30 @@ typedef struct {
         int      active[SQL_NSERVICE];
 } SQL_cfg;
 
-extern SQL_cfg sql_cfg;
+typedef struct {
+	char *query;
+	size_t ntuples;
+	size_t nfields;
+	SQL_TUPLE *tuple;
+} SQL_RESULT;
 
-int radiusd_sql_config();
-void radiusd_sql_shutdown();
-void radiusd_sql_clear_cache();
-void radiusd_sql_acct(grad_request_t *req);
-int radiusd_sql_checkgroup(grad_request_t *req, char *groupname);
-int radiusd_sql_check_attr_query(grad_request_t *req, grad_avp_t **check_pairs);
-int radiusd_sql_reply_attr_query(grad_request_t *req, grad_avp_t **reply_pairs);
-void radiusd_sql_auth_result_query(grad_request_t *req, int fail);
-void radiusd_sql_cleanup(int type, void *req);
-
-char *radiusd_sql_pass(grad_request_t *req, char *data);
+struct sql_connection {
+	SQL_cfg *cfg;
+	int    interface;        /* One of SQLT_ values */
+        int    type;             /* One of SQL_ values */
+        int    connected;        /* Connected to the database? */
+        int    destroy_on_close; /* Should the connection be closed upon
+				    the end of a transaction */
+        time_t last_used;        /* Time it was lastly used */
+        void   *data;            /* connection-specific data */
 	
-# ifdef RADIUS_SERVER_GUILE
-SCM sql_exec_query(int type, char *query);
-# endif
+	SQL_RESULT *cache[SQL_CACHE_SIZE];
+	size_t head;
+	size_t tail;
+};
 
 /* Dispatcher routines */
+void disp_init();
 int disp_sql_interface_index(char *name);
 int disp_sql_reconnect(int interface, int conn_type, struct sql_connection *conn);
 void disp_sql_disconnect(struct sql_connection *conn);
@@ -131,31 +118,4 @@ typedef struct {
 	int (*n_columns)(struct sql_connection *conn, void *data, size_t *np);
 } SQL_DISPATCH_TAB;
 
-# ifdef USE_SQL_MYSQL
-extern SQL_DISPATCH_TAB mysql_dispatch_tab[];
-# else
-#  define mysql_dispatch_tab NULL
-# endif
-# ifdef USE_SQL_POSTGRES
-extern SQL_DISPATCH_TAB postgres_dispatch_tab[];
-#else
-#  define postgres_dispatch_tab NULL
-#endif
-# ifdef USE_SQL_ODBC
-extern SQL_DISPATCH_TAB odbc_dispatch_tab[];
-# else
-#  define odbc_dispatch_tab NULL
-# endif
-
-#else
-# define sql_init()
-# define radiusd_sql_config() 0
-# define radiusd_sql_shutdown()
-# define radiusd_sql_clear_cache()
-# define radiusd_sql_acct(req)
-# define radiusd_sql_checkgroup(req, groupname) 0
-# define radiusd_sql_check_attr_query(req, check_pairs) 0
-# define radiusd_sql_reply_attr_query(req, reply_pairs)
-# define radiusd_sql_auth_result_query(req, fail)
-# define radiusd_sql_cleanup (void (*)(int, void *)) NULL
 #endif
