@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -27,8 +27,8 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <netdb.h>
-#include <radiusd.h>
-#include <argcv.h>
+#include <common.h>
+#include <radius/argcv.h>
 
 enum ascend_filter_type {
 	ascend_filter_generic,      /* 0 */
@@ -49,8 +49,8 @@ enum ascend_filter_cmp_op {
 
 /* The format of an IP filter attribute value. Fields are in netorder */
 typedef struct {
-	UINT4    src_ip;       /* The source IP address.  */
-	UINT4    dst_ip;       /* The destination IP address.  */
+	grad_uint32_t    src_ip;       /* The source IP address.  */
+	grad_uint32_t    dst_ip;       /* The destination IP address.  */
 	u_char   src_masklen;  /* The source netmask length */
 	u_char   dst_masklen;  /* The destination netmask length */
 	u_char   proto;        /* The IP protocol number */
@@ -66,7 +66,7 @@ typedef struct {
 /* IPX stuff. Not used at the moment */
 #define IPX_NODE_ADDR_LEN 6
 
-typedef UINT4   IPXADDR;
+typedef grad_uint32_t   IPXADDR;
 typedef char    IPXNODE[IPX_NODE_ADDR_LEN];
 typedef u_short IPXSOCKET;
 
@@ -120,23 +120,6 @@ struct ascend_parse_buf {
 	char **errmsg;   /* Error message */
 };
 
-static char *_get_token(struct ascend_parse_buf *pb, int require);
-static char *_lookahead(struct ascend_parse_buf *pb);
-static int _get_type(struct ascend_parse_buf *pb);
-static int _get_dir(struct ascend_parse_buf *pb);
-static int _get_action(struct ascend_parse_buf *pb);
-static int _get_hex_string(struct ascend_parse_buf *pb, u_char *buf);
-static int _ascend_parse_generic(struct ascend_parse_buf *pb);
-static int _get_protocol(struct ascend_parse_buf *pb);
-static int _get_direction_type(struct ascend_parse_buf *pb, char *suffix, int la);
-static int _get_ip(struct ascend_parse_buf *pb);
-static int _ascend_parse_ip_clause(struct ascend_parse_buf *pb);
-static int _get_op(struct ascend_parse_buf *pb);
-static int _get_port(struct ascend_parse_buf *pb);
-static int _ascend_parse_port_clause(struct ascend_parse_buf *pb);
-static int _ascend_parse_ip(struct ascend_parse_buf *pb);
-static int _ascend_parse(struct ascend_parse_buf *pb);
- 
 /* generic (more or less) calls */
 #define _moreinput(pb) ((pb)->tokn < (pb)->tokc)
 
@@ -220,7 +203,7 @@ _get_action(struct ascend_parse_buf *pb)
 /* ************************************************************************* */
 /* GENERIC filter parsing */
 
-int
+static int
 _get_hex_string(struct ascend_parse_buf *pb, u_char *buf)
 {
 	u_char tmp[2*ASCEND_MAX_CMP_LENGTH], *p;
@@ -270,7 +253,7 @@ _get_hex_string(struct ascend_parse_buf *pb, u_char *buf)
 	offset is number
 	mask and value are hex strings */
    
-int
+static int
 _ascend_parse_generic(struct ascend_parse_buf *pb)
 {
 	char *p;
@@ -319,7 +302,7 @@ _ascend_parse_generic(struct ascend_parse_buf *pb)
 		pb->flt->v.generic.more = 1;
 	else {
 		asprintf(pb->errmsg,
-			 _("Expected `more' but found `%s'"),
+			 _("Expected `more', but found `%s'"),
 			 tok);
 		return 1;
 	}
@@ -328,7 +311,7 @@ _ascend_parse_generic(struct ascend_parse_buf *pb)
 
 /* ************************************************************************* */
 /* IP filter parsing */
-int
+static int
 _get_protocol(struct ascend_parse_buf *pb)
 {
 	char *tok = _get_token(pb, 1);
@@ -356,7 +339,7 @@ _get_protocol(struct ascend_parse_buf *pb)
 #define ASCEND_DIR_SRC 0
 #define ASCEND_DIR_DST 1
 
-int
+static int
 _get_direction_type(struct ascend_parse_buf *pb, char *suffix, int lookahead)
 {
 	char *tok = lookahead ? _lookahead(pb) : _get_token(pb, 1);
@@ -371,23 +354,23 @@ _get_direction_type(struct ascend_parse_buf *pb, char *suffix, int lookahead)
 	}
 	if (!lookahead)
 		asprintf(pb->errmsg,
-			 _("Expected {src|dst}port but found `%s'"), tok);
+			 _("Expected `{src|dst}port', but found `%s'"), tok);
 	return ASCEND_DIR_NONE;
 }
 
-int
+static int
 _get_ip(struct ascend_parse_buf *pb)
 {
 	int dir = _get_direction_type(pb, "ip", 0);
 	char *tok;
-	UINT4 ip, mask;
+	grad_uint32_t ip, mask;
 	
 	if (dir == ASCEND_DIR_NONE)
 		return ASCEND_DIR_NONE;
 	tok = _get_token(pb, 1);
 	if (!tok)
 		return ASCEND_DIR_NONE;
-	ip = ip_strtoip(tok); /*FIXME: no error checking */
+	ip = grad_ip_strtoip(tok); /*FIXME: no error checking */
 
 	if (_moreinput(pb) && _lookahead(pb)[0] == '/') {
 		char *p;
@@ -423,7 +406,7 @@ _get_ip(struct ascend_parse_buf *pb)
 
 /* FIXME: if second {src|dst}ip is misspelled, the function returns
    success, supposing it was a portspec */
-int
+static int
 _ascend_parse_ip_clause(struct ascend_parse_buf *pb)
 {
 	int n;
@@ -444,7 +427,7 @@ _ascend_parse_ip_clause(struct ascend_parse_buf *pb)
 	return 0;
 }
 
-int
+static int
 _get_op(struct ascend_parse_buf *pb)
 {
 	char *s = _get_token(pb, 1);
@@ -465,7 +448,7 @@ _get_op(struct ascend_parse_buf *pb)
 	return ascend_cmp_none;
 }
 
-int
+static int
 _get_port(struct ascend_parse_buf *pb)
 {
 	int dir = _get_direction_type(pb, "port", 0);
@@ -522,24 +505,33 @@ _get_port(struct ascend_parse_buf *pb)
 	return dir;
 }
 
-int
+/* Return value:
+   0 - No port specification found
+   1 - Port specification is found and processed
+   -1 - Parse error pb->errmsg *might* contain diagnostics */
+static int
 _ascend_parse_port_clause(struct ascend_parse_buf *pb)
 {
-	int n = _get_port(pb);
+	int n;
+	
+	if (_get_direction_type(pb, "port", 1) == ASCEND_DIR_NONE)
+		return 0;
+	
+	n = _get_port(pb);
 
 	if (n == ASCEND_DIR_NONE)
-		return 1;
+		return -1;
 	if (_get_direction_type(pb, "port", 1) != ASCEND_DIR_NONE) {
 		int n1 = _get_port(pb);
 		if (n1 == ASCEND_DIR_NONE)
-			return 1;
+			return -1;
 		if (n1 == n) {
 			asprintf(pb->errmsg,
-				 _("Duplicate IP specification"));
-			return 1;
+				 _("Duplicate port specification"));
+			return -1;
 		}
 	}
-	return 0;
+	return 1;
 }
 
 /* IP filter specification is
@@ -556,7 +548,7 @@ _ascend_parse_port_clause(struct ascend_parse_buf *pb)
 	 PROTO  is either the protocol number or its name from /etc/protocols
 	 PORT   is either the port number or its name from /etc/services */
 
-int
+static int
 _ascend_parse_ip(struct ascend_parse_buf *pb)
 {
 	if (!_moreinput(pb))
@@ -570,7 +562,8 @@ _ascend_parse_ip(struct ascend_parse_buf *pb)
 			return 1;
 		if (_moreinput(pb)) {
 			char *tok;
-			if (_ascend_parse_port_clause(pb))
+			int have_port = _ascend_parse_port_clause(pb);
+			if (have_port == -1)
 				return 1;
 			tok = _get_token(pb, 0);
 			if (!tok)
@@ -579,7 +572,9 @@ _ascend_parse_ip(struct ascend_parse_buf *pb)
 				pb->flt->v.ip.established = 1;
 			else {
 				asprintf(pb->errmsg,
-					 _("Expected `est' but found `%s'"),
+					 have_port ?
+					 _("Expected `est' but found `%s'") :
+					 _("Expected `{src|dst}port' or `est', but found `%s'"),
 					 tok);
 				return 1;
 			}
@@ -597,14 +592,14 @@ _ascend_parse_ip(struct ascend_parse_buf *pb)
                      [ "srcipxsoc" cmp HEXNUM ]]
                     [ "dstipxnet" NETADDR "dstipxnode" NODE
   		     [ "dstipxsoc" cmp HEXNUM ]] */
-int
+static int
 _ascend_parse_ipx(struct ascend_parse_buf *pb)
 {
 	asprintf(pb->errmsg, "IPX filters are not yet supported");
 	return 1;
 }
  
-int
+static int
 _ascend_parse(struct ascend_parse_buf *pb)
 {
 	memset(pb->flt, 0, sizeof(pb->flt[0]));
@@ -628,7 +623,7 @@ _ascend_parse(struct ascend_parse_buf *pb)
    Return 0 and fill flt[0] if the specification is correct.
    Return !0 and return diagnostics in errp otherwise.
    NOTE: errp is malloced and should be freed using usual free() */
-int
+static int
 _ascend_parse_filter(const char *input, ASCEND_FILTER *flt, char **errp)
 {
 	struct ascend_parse_buf pb;
@@ -652,15 +647,15 @@ _ascend_parse_filter(const char *input, ASCEND_FILTER *flt, char **errp)
 }
 
 int
-ascend_parse_filter(VALUE_PAIR *pair, char **errp)
+grad_ascend_parse_filter(grad_avp_t *pair, char **errp)
 {
 	ASCEND_FILTER flt;
 	
 	if (_ascend_parse_filter(pair->avp_strvalue, &flt, errp)) 
 		return 1;
-	efree(pair->avp_strvalue);
+	grad_free(pair->avp_strvalue);
 	pair->avp_strlength = sizeof(flt);
-	pair->avp_strvalue = emalloc(pair->avp_strlength);
+	pair->avp_strvalue = grad_emalloc(pair->avp_strlength);
 	memcpy(pair->avp_strvalue, &flt, sizeof(flt));
 	return 0;
 }
