@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
  
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_CLIENT_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -33,19 +33,18 @@
 #include <errno.h>
 #include <ctype.h>
 
-#include <radius.h>
-#include <debugmod.h>
+#include <common.h>
 
 void
-rad_clt_random_vector(char *vector)
+grad_client_random_authenticator(char *authenticator)
 {
         int randno;
         int i;
 
-        for (i = 0; i < AUTH_VECTOR_LEN; ) {
+        for (i = 0; i < GRAD_AUTHENTICATOR_LENGTH; ) {
                 randno = rand();
-                memcpy(vector, &randno, sizeof(int));
-                vector += sizeof(int);
+                memcpy(authenticator, &randno, sizeof(int));
+                authenticator += sizeof(int);
                 i += sizeof(int);
         }
 }
@@ -53,9 +52,9 @@ rad_clt_random_vector(char *vector)
 #define PERM S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP
 
 unsigned
-rad_clt_message_id(RADIUS_SERVER *server)
+grad_client_message_id(grad_server_t *server)
 {
-	SERVER_ID sid;
+	grad_server_id_t sid;
 	int fd;
 	unsigned id;
 	
@@ -66,19 +65,21 @@ rad_clt_message_id(RADIUS_SERVER *server)
 		fstat(fd, &st);
 		if (server->id_offset != (off_t) -1
 		    && server->id_offset + sizeof(sid) <= st.st_size) {
-			rad_lock(fd, sizeof(sid), server->id_offset, SEEK_SET);
+			grad_lock_file(fd, sizeof(sid),
+				       server->id_offset, SEEK_SET);
 			lseek(fd, server->id_offset, SEEK_SET);
 			read(fd, &sid, sizeof(sid));
 			id = sid.id++;
 			lseek(fd, server->id_offset, SEEK_SET);
 			write(fd, &sid, sizeof(sid));
-			rad_unlock(fd, sizeof(sid),
-				   server->id_offset, SEEK_SET);
+			grad_unlock_file(fd, sizeof(sid),
+					 server->id_offset, SEEK_SET);
 
 		} else {
 			off_t off = 0;
 			lseek(fd, 0, SEEK_SET);
-			rad_lock(fd, st.st_size + sizeof(sid), 0, SEEK_SET);
+			grad_lock_file(fd, st.st_size + sizeof(sid),
+				       0, SEEK_SET);
 			while (read(fd, &sid, sizeof(sid)) == sizeof(sid)) {
 				if (sid.addr == server->addr) {
 					id = sid.id++;
@@ -96,7 +97,8 @@ rad_clt_message_id(RADIUS_SERVER *server)
 				server->id_offset = off;
 				id = 0;
 			} 
-			rad_unlock(fd, st.st_size + sizeof(sid), 0, SEEK_SET);
+			grad_unlock_file(fd, st.st_size + sizeof(sid),
+					 0, SEEK_SET);
 		}
 		close(fd);
 	} else {
@@ -105,62 +107,98 @@ rad_clt_message_id(RADIUS_SERVER *server)
 	return id;
 }
 	
-RADIUS_REQ *
-rad_clt_recv(UINT4 host, u_short udp_port, char *secret, char *vector,
+grad_request_t *
+grad_client_recv(grad_uint32_t host, u_short udp_port, char *secret, char *authenticator,
 	     char *buffer, int length)
 {
-        AUTH_HDR *auth;
+        grad_packet_header_t *auth;
         int totallen;
-        u_char reply_digest[AUTH_VECTOR_LEN];
-        u_char calc_digest[AUTH_VECTOR_LEN];
+        u_char reply_digest[GRAD_AUTHENTICATOR_LENGTH];
+        u_char calc_digest[GRAD_AUTHENTICATOR_LENGTH];
         int  secretlen;
-
-        auth = (AUTH_HDR *)buffer;
+	grad_request_t *req;
+	
+        auth = (grad_packet_header_t *)buffer;
         totallen = ntohs(auth->length);
 
         if (totallen != length) {
-                radlog(L_ERR,
+                grad_log(L_ERR,
            _("Actual request length does not match reported length (%d, %d)"),
-                       totallen, length);
+                         totallen, length);
                 return NULL;
         }
 
         /* Verify the reply digest */
         secretlen = strlen(secret);
-        memcpy(reply_digest, auth->vector, AUTH_VECTOR_LEN);
-        memcpy(auth->vector, vector, AUTH_VECTOR_LEN);
+        memcpy(reply_digest, auth->authenticator, GRAD_AUTHENTICATOR_LENGTH);
+        memcpy(auth->authenticator, authenticator, GRAD_AUTHENTICATOR_LENGTH);
         memcpy(buffer + length, secret, secretlen);
-        md5_calc(calc_digest, (unsigned char *)auth, length + secretlen);
+        grad_md5_calc(calc_digest, (unsigned char *)auth, length + secretlen);
         
-	debug(1, ("received %s", auth_code_str(auth->code)));
-        if (memcmp(reply_digest, calc_digest, AUTH_VECTOR_LEN) != 0) {
-                radlog(L_WARN, _("Received invalid reply digest from server"));
+	debug(1, ("received %s", grad_request_code_to_name(auth->code)));
+        if (memcmp(reply_digest, calc_digest, GRAD_AUTHENTICATOR_LENGTH) != 0) {
+                grad_log(L_WARN, _("Received invalid reply digest from server"));
         }
 
-        return rad_decode_pdu(host, udp_port, buffer, length);
+        req = grad_decode_pdu(host, udp_port, buffer, length);
+	req->secret = secret;
+		
+	return req;
 }
 
-static VALUE_PAIR *
-_encode_pairlist(VALUE_PAIR *p, u_char *vector, u_char *secret)
+grad_avp_t *
+grad_client_encrypt_pairlist(grad_avp_t *plist, u_char *authenticator, u_char *secret)
 {
-	VALUE_PAIR *ret = avl_dup(p);
-
-	for (p = ret; p; p = p->next)
-		if (p->attribute == DA_USER_PASSWORD
-		    || p->attribute == DA_CHAP_PASSWORD) {
+	grad_avp_t *p;
+	
+	for (p = plist; p; p = p->next) {
+		if (p->prop & GRAD_AP_ENCRYPT_RFC2138) {
 			char *pass = p->avp_strvalue;
-			encrypt_password(p, pass, vector, secret);
-			efree(pass);
+			grad_encrypt_password(p, pass, authenticator, secret);
+			grad_free(pass);
+		} else if (p->prop & GRAD_AP_ENCRYPT_RFC2868) {
+			char *pass = p->avp_strvalue;
+			grad_encrypt_tunnel_password(p, 0, pass,
+						     authenticator, secret);
+			grad_free(pass);
 		}
-	return ret;
+	}
+	return plist;
+}	
+
+grad_avp_t *
+grad_client_decrypt_pairlist(grad_avp_t *plist, u_char *authenticator, u_char *secret)
+{
+	grad_avp_t *p;
+	char password[GRAD_STRING_LENGTH+1];
+	
+	for (p = plist; p; p = p->next) {
+		if (p->prop & GRAD_AP_ENCRYPT_RFC2138) {
+			grad_decrypt_password(password, p, authenticator, secret);
+			grad_free(p->avp_strvalue);
+			p->avp_strvalue = grad_estrdup(password);
+			p->avp_strlength = strlen(p->avp_strvalue);
+		} else if (p->prop & GRAD_AP_ENCRYPT_RFC2868) {
+			u_char tag;
+			
+			grad_decrypt_tunnel_password(password,
+						     &tag,
+						     p,
+						     authenticator,
+						     secret);
+			grad_free(p->avp_strvalue);
+			p->avp_strvalue = grad_estrdup(password);
+			p->avp_strlength = strlen(p->avp_strvalue);
+		}
+	}
+	return plist;
 }
 
-
-RADIUS_REQ *
-rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
-	      VALUE_PAIR *pairlist,
-	      int flags,
-	      int *authid, u_char *authvec)
+grad_request_t *
+grad_client_send0(grad_server_queue_t *config, int port_type, int code,
+		  grad_avp_t *pairlist,
+		  int flags,
+		  int *authid, u_char *authvec)
 {
 	struct sockaddr salocal;
 	struct sockaddr saremote;
@@ -169,20 +207,20 @@ rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
         int sockfd;
         int salen;
         int i;
-        RADIUS_REQ *req = NULL;
-        RADIUS_SERVER *server;
-        char ipbuf[DOTTED_QUAD_LEN];
+        grad_request_t *req = NULL;
+        grad_server_t *server;
+        char ipbuf[GRAD_IPV4_STRING_LENGTH];
 	char *recv_buf;
-	ITERATOR *itr;
+	grad_iterator_t *itr;
 	int id;
 	
         if (port_type < 0 || port_type > 2) {
-                radlog(L_ERR, _("invalid port type"));
+                grad_log(L_ERR, _("invalid port type"));
                 return NULL;
         }
-        sockfd = socket(AF_INET, SOCK_DGRAM, 0);
+        sockfd = socket(PF_INET, SOCK_DGRAM, 0);
         if (sockfd < 0) {
-                radlog(L_ERR|L_PERROR, "socket");
+                grad_log(L_ERR|L_PERROR, "socket");
                 return NULL;
         }
 
@@ -200,55 +238,56 @@ rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
         } while ((bind(sockfd, &salocal, sizeof (struct sockaddr_in)) < 0)
 		 && local_port < 65535);
         if (local_port >= 65535) {
-                radlog(L_ERR|L_PERROR, "bind");
+                grad_log(L_ERR|L_PERROR, "bind");
                 close(sockfd);
                 return NULL;
         }
 
 	debug(1,
-	      ("sending %s", auth_code_str(code)));
-	recv_buf = emalloc(config->buffer_size);
-        itr = iterator_create(config->servers);
-        server = iterator_first(itr);
+	      ("sending %s", grad_request_code_to_name(code)));
+	recv_buf = grad_emalloc(config->buffer_size);
+        itr = grad_iterator_create(config->servers);
+        server = grad_iterator_first(itr);
         do {
 		fd_set readfds;
 		struct timeval tm;
 		int result;
-		u_char vector[AUTH_VECTOR_LEN];
+		u_char authenticator[GRAD_AUTHENTICATOR_LENGTH];
 		void *pdu;
 		size_t size;
-		VALUE_PAIR *pair;
+		grad_avp_t *pair;
 
                 if (server->port[port_type] <= 0)
                         continue;
                 
                 if (debug_on(10)) {
-                        radlog(L_DEBUG, "server %s:%d",
-                               ip_iptostr(server->addr, ipbuf),
-                               server->port[port_type]);
+                        grad_log(L_DEBUG, "server %s:%d",
+                                 grad_ip_iptostr(server->addr, ipbuf),
+                                 server->port[port_type]);
                 }
 
                 if (authid && (flags & RADCLT_AUTHENTICATOR))
-			memcpy(vector, authvec, sizeof vector);
+			memcpy(authenticator, authvec, sizeof authenticator);
 		else
-			rad_clt_random_vector(vector);
+			grad_client_random_authenticator(authenticator);
 		if (authid && (flags & RADCLT_ID))
 			id = *authid;
 		else
-			id = rad_clt_message_id(server);
-		pair = _encode_pairlist(pairlist, vector, server->secret);
-		size = rad_create_pdu(&pdu, code,
-				      id,
-				      vector,
-				      server->secret,
-				      pair,
-				      NULL);
+			id = grad_client_message_id(server);
+		pair = grad_client_encrypt_pairlist(grad_avl_dup(pairlist),
+						    authenticator, server->secret);
+		size = grad_create_pdu(&pdu, code,
+				       id,
+				       authenticator,
+				       server->secret,
+				       pair,
+				       NULL);
 		if (authid && !(flags & RADCLT_ID))
 			*authid = id;
 		if (authvec && !(flags & RADCLT_AUTHENTICATOR))
-			memcpy(authvec, vector, sizeof vector);
+			memcpy(authvec, authenticator, sizeof authenticator);
 		
-		avl_free(pair);
+		grad_avl_free(pair);
 		
                 if (size <= 0) 
                         break; /*FIXME: continue anyway?*/
@@ -265,7 +304,7 @@ rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
                         if (sendto(sockfd, pdu, size, 0,
                                    &saremote,
                                    sizeof(struct sockaddr_in)) == -1) {
-                                radlog(L_ERR|L_PERROR, "sendto");
+                                grad_log(L_ERR|L_PERROR, "sendto");
                         }
 
                         salen = sizeof (saremote);
@@ -281,7 +320,7 @@ rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
 					      ("select interrupted. retrying."));
                                         continue;
                                 }
-                                radlog(L_NOTICE, _("select() interrupted"));
+                                grad_log(L_NOTICE, _("select() interrupted"));
                                 break;
                         }
 
@@ -292,44 +331,44 @@ rad_clt_send0(RADIUS_SERVER_QUEUE *config, int port_type, int code,
                                                   0, &saremote, &salen);
 
                                 if (result > 0) 
-                                        req = rad_clt_recv(
+                                        req = grad_client_recv(
 						sin->sin_addr.s_addr,
                                                 sin->sin_port,
                                                 server->secret,
-                                                vector,
+                                                authenticator,
                                                 recv_buf,
                                                 result);
                                 else 
-                                        radlog(L_ERR|L_PERROR,
+                                        grad_log(L_ERR|L_PERROR,
                                         _("error receiving data from %s:%d"),
-                                               ip_iptostr(server->addr, ipbuf),
-                                               server->port[port_type]);
+                                                 grad_ip_iptostr(server->addr, ipbuf),
+                                                 server->port[port_type]);
                                 
                                 break;
                         }
 			debug(10,("no response. retrying."));
                 }
 		
-		efree(pdu);
+		grad_free(pdu);
 		
                 if (!req)
                         debug(10,("no reply from %s:%d",
-				  ip_iptostr(server->addr, ipbuf),
+				  grad_ip_iptostr(server->addr, ipbuf),
 				  server->port[port_type]));
 		
-        } while (!req && (server = iterator_next(itr)) != NULL);
-	iterator_destroy(&itr);
+        } while (!req && (server = grad_iterator_next(itr)) != NULL);
+	grad_iterator_destroy(&itr);
 
-	efree(recv_buf);
+	grad_free(recv_buf);
         close(sockfd);
         return req;
 }
 
-RADIUS_REQ *
-rad_clt_send(RADIUS_SERVER_QUEUE *config, int port_type, int code,
-	     VALUE_PAIR *pairlist)
+grad_request_t *
+grad_client_send(grad_server_queue_t *config, int port_type, int code,
+		 grad_avp_t *pairlist)
 {
-	return rad_clt_send0(config, port_type, code, pairlist, 0, NULL, NULL);
+	return grad_client_send0(config, port_type, code, pairlist, 0, NULL, NULL);
 }
 
 /* ************************************************************************* */
@@ -341,7 +380,7 @@ rad_clt_send(RADIUS_SERVER_QUEUE *config, int port_type, int code,
 #define TOK_TIMEOUT    3
 #define TOK_RETRY      4
 
-static struct keyword kwd[] = {
+static grad_keyword_t kwd[] = {
         { "source_ip", TOK_SOURCE_IP },
         { "source-ip", TOK_SOURCE_IP },
         { "server", TOK_SERVER },
@@ -351,35 +390,33 @@ static struct keyword kwd[] = {
 };
 
 static int
-parse_client_config(RADIUS_SERVER_QUEUE *client, int argc, char **argv,
-		    char *file, int lineno)
+parse_client_config(void *closure, int argc, char **argv, grad_locus_t *loc)
 {
+	grad_server_queue_t *client = closure;
         char *p;
-        RADIUS_SERVER serv;
+        grad_server_t serv;
         
-        switch (xlat_keyword(kwd, argv[0], TOK_INVALID)) {
+        switch (grad_xlat_keyword(kwd, argv[0], TOK_INVALID)) {
         case TOK_INVALID:
-                radlog(L_ERR, "%s:%d: unknown keyword", file, lineno);
+                grad_log_loc(L_ERR, loc, _("unknown keyword"));
                 break;
                 
         case TOK_SOURCE_IP:
-                client->source_ip = ip_gethostaddr(argv[1]);
+                client->source_ip = grad_ip_gethostaddr(argv[1]);
                 break;
                 
         case TOK_SERVER:
                 if (argc != 6) {
-                        radlog(L_ERR, "%s:%d: wrong number of fields",
-                               file, lineno);
+                        grad_log_loc(L_ERR, loc, _("wrong number of fields"));
                         break;
                 }
                 memset(&serv, 0, sizeof serv);
 
                 serv.name = argv[1];
-                serv.addr = ip_gethostaddr(argv[2]);
+                serv.addr = grad_ip_gethostaddr(argv[2]);
                 if (!serv.addr) {
-                        radlog(L_ERR,
-                               "%s:%d: bad IP address or host name",
-                               file, lineno);
+                        grad_log_loc(L_ERR, loc,
+				     _("bad IP address or host name"));
                         break;
                 }
                 
@@ -387,50 +424,45 @@ parse_client_config(RADIUS_SERVER_QUEUE *client, int argc, char **argv,
 
                 serv.port[0] = strtol(argv[4], &p, 0);
                 if (*p) {
-                        radlog(L_ERR,
-                               "%s:%d: bad port number %s",
-                               file, lineno, argv[4]);
+                        grad_log_loc(L_ERR, loc, _("bad port number %s"),
+				     argv[4]);
                         break;
                 }
 
                 serv.port[1] = strtol(argv[5], &p, 0);
                 if (*p) {
-                        radlog(L_ERR,
-                               "%s:%d: bad port number %s",
-                               file, lineno, argv[4]);
+                        grad_log_loc(L_ERR, loc, _("bad port number %s"),
+				     argv[4]);
                         break;
                 }
 
-		rad_clt_append_server(client, rad_clt_alloc_server(&serv));
+		grad_client_append_server(client,
+					  grad_client_alloc_server(&serv));
                 break;
                 
         case TOK_TIMEOUT:
                 client->timeout = strtol(argv[1], &p, 0);
-                if (*p) {
-                        radlog(L_ERR,
-                               "%s:%d: bad timeout value", file, lineno);
-                }
+                if (*p) 
+                        grad_log_loc(L_ERR, loc,  _("bad timeout value"));
                 break;
                 
         case TOK_RETRY:
                 client->retries = strtol(argv[1], &p, 0);
-                if (*p) {
-                        radlog(L_ERR,
-                               "%s:%d: bad retry value", file, lineno);
-                }
+                if (*p) 
+                        grad_log_loc(L_ERR, loc, _("bad retry value"));
                 break;
         }
         return 0;
 }
 
 
-RADIUS_SERVER_QUEUE *
-rad_clt_create_queue(int read_cfg, UINT4 source_ip, size_t bufsize)
+grad_server_queue_t *
+grad_client_create_queue(int read_cfg, grad_uint32_t source_ip, size_t bufsize)
 {
-        RADIUS_SERVER_QUEUE *client;
+        grad_server_queue_t *client;
         char *filename;
         
-        client = emalloc(sizeof *client);
+        client = grad_emalloc(sizeof *client);
 
         /* Provide default values */
         client->source_ip = source_ip;
@@ -440,48 +472,49 @@ rad_clt_create_queue(int read_cfg, UINT4 source_ip, size_t bufsize)
         client->servers = 0;
 
         if (read_cfg) {
-                filename = mkfilename(radius_dir, "client.conf");
-                read_raddb_file(filename, 1, parse_client_config, client);
-                efree(filename);
+                filename = grad_mkfilename(radius_dir, "client.conf");
+                grad_read_raddb_file(filename, 1, NULL,
+                                     parse_client_config, client);
+                grad_free(filename);
         }
         return client;
 }
 
 void
-rad_clt_destroy_queue(RADIUS_SERVER_QUEUE *queue)
+grad_client_destroy_queue(grad_server_queue_t *queue)
 {
 	if (queue) {
-		rad_clt_clear_server_list(queue);
-		efree(queue);
+		grad_client_clear_server_list(queue);
+		grad_free(queue);
 	}
 }
 
-RADIUS_SERVER *
-rad_clt_alloc_server(RADIUS_SERVER *src)
+grad_server_t *
+grad_client_alloc_server(grad_server_t *src)
 {
-        RADIUS_SERVER *server;
+        grad_server_t *server;
 
-        server = emalloc(sizeof(*server));
-        server->name = estrdup(src->name);
+        server = grad_emalloc(sizeof(*server));
+        server->name = grad_estrdup(src->name);
         server->addr = src->addr;
         server->port[0] = src->port[0];
         server->port[1] = src->port[1];
-        server->secret = estrdup(src->secret);
+        server->secret = grad_estrdup(src->secret);
 	server->id_offset = (off_t)-1;
         return server;
 }
 
-RADIUS_SERVER *
-rad_clt_dup_server(RADIUS_SERVER *src)
+grad_server_t *
+grad_client_dup_server(grad_server_t *src)
 {
-        RADIUS_SERVER *dest;
+        grad_server_t *dest;
 
-        dest = emalloc(sizeof(*dest));
+        dest = grad_emalloc(sizeof(*dest));
         dest->addr = src->addr;
-        dest->name = estrdup(src->name);
+        dest->name = grad_estrdup(src->name);
         dest->port[0] = src->port[0];
         dest->port[1] = src->port[1];
-        dest->secret = estrdup(src->secret);
+        dest->secret = grad_estrdup(src->secret);
         return dest;
 }
 
@@ -490,49 +523,50 @@ rad_clt_dup_server(RADIUS_SERVER *src)
  */
 
 void
-rad_clt_free_server(RADIUS_SERVER *server)
+grad_client_free_server(grad_server_t *server)
 {
-        efree(server->name);
-        efree(server->secret);
-        efree(server);
+        grad_free(server->name);
+        grad_free(server->secret);
+        grad_free(server);
 }
 
 void
-rad_clt_append_server(RADIUS_SERVER_QUEUE *qp, RADIUS_SERVER *server)
+grad_client_append_server(grad_server_queue_t *qp, grad_server_t *server)
 {
 	if (!qp->servers)
-		qp->servers = list_create();
-	list_append(qp->servers, server);
+		qp->servers = grad_list_create();
+	grad_list_append(qp->servers, server);
 }
 
 static int
-rad_clt_internal_free_server(void *item, void *data)
+grad_client_internal_free_server(void *item, void *data)
 {
-	RADIUS_SERVER *server = item;
-        efree(server->name);
-        efree(server->secret);
-	efree(server);
+	grad_server_t *server = item;
+        grad_free(server->name);
+        grad_free(server->secret);
+	grad_free(server);
 	return 0;
 }
 
 void
-rad_clt_clear_server_list(RADIUS_SERVER_QUEUE *qp)
+grad_client_clear_server_list(grad_server_queue_t *qp)
 {
-	list_destroy(&qp->servers, rad_clt_internal_free_server, NULL);
+	grad_list_destroy(&qp->servers,
+			  grad_client_internal_free_server, NULL);
 }
 
 static int
 server_cmp(const void *item, const void *data)
 {
-	const RADIUS_SERVER *serv = item;
+	const grad_server_t *serv = item;
 	const char *id = data;
 
         return strcmp(serv->name, id);
 }
 
-RADIUS_SERVER *
-rad_clt_find_server(RADIUS_SERVER_QUEUE *qp, char *name)
+grad_server_t *
+grad_client_find_server(grad_server_queue_t *qp, char *name)
 {
-	return list_locate(qp->servers, name, server_cmp);
+	return grad_list_locate(qp->servers, name, server_cmp);
 }
 
