@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -23,7 +23,8 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <grp.h>
-#include <mem.h>
+
+#include <radius/mem.h>
 
 LOCK_DECLARE(lock)
 
@@ -49,7 +50,7 @@ store_group(struct group *grp)
 	for (i = 0; i < grcnt; i++)
 		len += strlen(grp->gr_mem[i]) + 1;
 
-	result = emalloc(len);
+	result = grad_emalloc(len);
 	*result = *grp;
 	result->gr_mem = (char**)(result + 1);
 	buffer = (char*)(result->gr_mem + grcnt + 1);
@@ -70,7 +71,7 @@ store_group(struct group *grp)
 }
 	
 struct group *
-rad_getgrnam(const char *name)
+grad_getgrnam(const char *name)
 {
 	struct group *grp;
 
