@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius
-   Copyright (C) 2000,2001,2002,2003 Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
   
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_CHECKRAD_C
-
 #if defined(HAVE_CONFIG_H)
 # include <config.h>
 #endif
@@ -39,97 +39,144 @@
 #endif
 #include <signal.h>
 
-#include <obstack1.h>
 #include <radiusd.h>
-#include <radutmp.h>
+#include <radius/radutmp.h>
 #include <checkrad.h>
+#include <rewrite.h>
 
-#include <asn1.h>
-#include <snmp.h>
+#include <snmp/asn1.h>
+#include <snmp/snmp.h>
 
 struct check_instance {
-        char      *name;
-        int       port;
-        char      *sid;
-        UINT4     ip;
-        int       result;
-        int       timeout;
-        int       method;
-        char      *func;
-        envar_t   *args;
-        char      *hostname;
+	/* Data from the radutmp entry */
+        char          *name;       /* User name */
+	grad_uint32_t nas_ip;      /* NAS IP address */
+        int           port;        /* NAS port number */
+        char          *sid;        /* Session ID */
+        grad_uint32_t framed_ip;   /* Framed IP address */
+
+	/* Data from the nastypes entry */
+	int           method;      /* Method to be used */
+        char          *func;       /* Name of the rewrite function */
+        grad_envar_t  *args;       /* Method arguments */
+
+	/* Additional data */
+        char          *nasname;    /* Name of the NAS (for diagnostics) */
+	struct obstack stack;      /* Memory allocation stack */
+
+	/* Results */
+        int           timeout;     /* Was the timeout hit? */
+        int           result;      /* Result of the test */
 };
 
 char *
 slookup(struct check_instance *checkp, char *name, char *defval)
 {
-        return envar_lookup_str(checkp->args, name, defval);
+        return grad_envar_lookup_str(checkp->args, name, defval);
 }
 
 int
 ilookup(struct check_instance *checkp, char *name, int defval)
 {
-        return envar_lookup_int(checkp->args, name, defval);
+        return grad_envar_lookup_int(checkp->args, name, defval);
 }
         
 struct check_instance *
-create_instance(struct check_instance *cptr, NAS *nas, struct radutmp *up)
+create_instance(struct check_instance *cptr, grad_nas_t *nas, struct radutmp *up)
 {
         RADCK_TYPE *radck_type;
                 
         if ((radck_type = find_radck_type(nas->nastype)) == NULL) {
-                radlog(L_ERR,
-                       _("unknown NAS type: %s (nas %s)"),
-                       nas->nastype,
-                       nas->shortname);
+                grad_log(L_ERR,
+                         _("unknown NAS type: %s (nas %s)"),
+                         nas->nastype,
+                         nas->shortname);
                 return NULL;
         }
-        cptr->name = up->orig_login;
+        cptr->name = up->orig_login[0] ? up->orig_login : up->login;
         cptr->port = up->nas_port;
         cptr->sid  = up->session_id;
-        cptr->ip   = up->framed_address;
-        cptr->result = -1;
+        cptr->framed_ip = up->framed_address;
+	cptr->nas_ip = up->nas_address;
+	cptr->result = -1;
         cptr->timeout = 0;
-        cptr->hostname = nas->shortname ? nas->shortname : nas->longname;
+        cptr->nasname = nas->shortname ? nas->shortname : nas->longname;
         
         cptr->method = radck_type->method;
-        cptr->args = envar_merge_lists((envar_t*) nas->args, radck_type->args);
+        cptr->args = grad_envar_merge_lists((grad_envar_t*) nas->args,
+					    radck_type->args);
         cptr->func = slookup(cptr, "function", NULL);
+	obstack_init(&cptr->stack);
         return cptr;
 }
 
 void
 free_instance(struct check_instance *cptr)
 {
-        envar_free_list(&cptr->args);
+	obstack_free(&cptr->stack, NULL);
+        grad_envar_free_list(&cptr->args);
 }
 
 int
 compare(struct check_instance *checkp, char *str)
 {
-        return va_run_init(checkp->func, NULL,
-                           "ssis",
-                           str,
-                           checkp->name,
-                           checkp->port,
-                           checkp->sid);
+	grad_uint32_t result;
+
+	if (rewrite_invoke(Integer, &result,
+			   checkp->func, NULL,
+			   "ssis",
+			   str,
+			   checkp->name,
+			   checkp->port,
+			   checkp->sid))
+		return -1;
+	return result;
 }
 
 
-/* Replace: %u  -- username
+char *
+checkrad_xlat_new(struct check_instance *checkp, char *template)
+{
+	grad_request_t *req;
+	char *str;
+	
+	/* Create a temporary request */
+	req = grad_request_alloc();
+	grad_avl_add_pair(&req->avlist,
+			  grad_avp_create_string(DA_USER_NAME, checkp->name));
+	grad_avl_add_pair(&req->avlist,
+			  grad_avp_create_integer(DA_NAS_PORT_ID,
+						  checkp->port));
+	grad_avl_add_pair(&req->avlist,
+			  grad_avp_create_string(DA_ACCT_SESSION_ID,
+						 checkp->sid));
+	grad_avl_add_pair(&req->avlist,
+			  grad_avp_create_integer(DA_FRAMED_IP_ADDRESS,
+						  checkp->framed_ip));
+	grad_avl_add_pair(&req->avlist,
+			  grad_avp_create_integer(DA_NAS_IP_ADDRESS,
+						  checkp->nas_ip));
+
+	str = util_xlate(&checkp->stack, template, req);
+
+	grad_request_free(req);
+	
+	return str;
+}
+
+/* Translate traditional OID specs.
+ * Replace: %u  -- username
  *          %s  -- session id
  *          %p  -- port no
  *          %P  -- port no + 1
  */
 char *
-checkrad_xlat(struct check_instance *checkp, char *str)
+checkrad_xlat_old(struct check_instance *checkp, char *str)
 {
         char *ptr;
         int len;
         char buf[24];
-        struct obstack stk;
 
-        obstack_init(&stk);
         while (*str) {
                 if (*str == '%') {
                         switch (str[1]) {
@@ -155,27 +202,34 @@ checkrad_xlat(struct check_instance *checkp, char *str)
                                 ptr = buf;
                                 break;
                         case 'i':
-                                ip_iptostr(checkp->ip, buf);
+                                grad_ip_iptostr(checkp->framed_ip, buf);
                                 ptr = buf;
                                 break;
                         default:
                                 ptr = NULL;
-                                obstack_grow(&stk, str, 2);
+                                obstack_grow(&checkp->stack, str, 2);
                         }
                         if (ptr) {
                                 len = strlen(ptr);
-                                obstack_grow(&stk, ptr, len);
+                                obstack_grow(&checkp->stack, ptr, len);
                         }
                         str += 2;
                 } else {
-                        obstack_1grow(&stk, *str);
+                        obstack_1grow(&checkp->stack, *str);
                         str++;
                 }
         }
-        obstack_1grow(&stk, 0);
-        str = estrdup(obstack_finish(&stk));
-        obstack_free(&stk, NULL);
-        return str;
+        obstack_1grow(&checkp->stack, 0);
+        return obstack_finish(&checkp->stack);
+}
+
+char *
+checkrad_xlat(struct check_instance *checkp, char *str)
+{
+	if (str[0] == '=')
+		return checkrad_xlat_new(checkp, str+1);
+	
+	return checkrad_xlat_old(checkp, str);
 }
 
 /*ARGSUSED*/
@@ -188,9 +242,9 @@ converse(int type, struct snmp_session *sp, struct snmp_pdu *pdu, void *closure)
         char buf[64];
 
         if (type == SNMP_CONV_TIMEOUT) {
-                radlog(L_NOTICE,
-                       _("timed out in waiting SNMP response from NAS %s"),
-                       checkp->hostname);
+                grad_log(L_NOTICE,
+                         _("timed out in waiting SNMP response from NAS %s"),
+                         checkp->nasname);
                 checkp->timeout++;
                 return 1;
         }
@@ -211,10 +265,10 @@ converse(int type, struct snmp_session *sp, struct snmp_pdu *pdu, void *closure)
                         debug(2, ("(INT) %d: %d", vlist->var_int, rc));
                         break;
                 case SMI_IPADDRESS:
-                        ip_iptostr(*(UINT4*)vlist->var_int, buf);
+                        grad_ip_iptostr(*(grad_uint32_t*)vlist->var_int, buf);
                         rc = compare(checkp, buf);
                         debug(2, ("(IPADDR) %#x: %d",
-                                  *(UINT4*)vlist->var_str, rc));
+                                  *(grad_uint32_t*)vlist->var_str, rc));
                         break;
                 }
 
@@ -224,7 +278,7 @@ converse(int type, struct snmp_session *sp, struct snmp_pdu *pdu, void *closure)
 }
 
 int
-snmp_check(struct check_instance *checkp, NAS *nas)
+snmp_check(struct check_instance *checkp, grad_nas_t *nas)
 {
         int rc = -1;
         struct snmp_pdu *pdu;
@@ -239,15 +293,16 @@ snmp_check(struct check_instance *checkp, NAS *nas)
         char *snmp_oid;
         
         if ((snmp_oid = slookup(checkp, "oid", NULL)) == NULL) {
-                radlog(L_ERR, _("no snmp_oid"));
+                grad_log(L_ERR, _("no snmp_oid"));
                 return -1;
         }
         snmp_oid = checkrad_xlat(checkp, snmp_oid);
+	if (!snmp_oid) /* The diagnostics has already been issued */
+		return -1;
         oid = oid_create_from_string(snmp_oid);
         if (!oid) {
-                radlog(L_ERR,
-                       _("invalid OID: %s"), snmp_oid);
-                efree(snmp_oid);
+                grad_log(L_ERR,
+                         _("invalid OID: %s"), snmp_oid);
                 return -1;
         }
         
@@ -263,38 +318,34 @@ snmp_check(struct check_instance *checkp, NAS *nas)
         sp = snmp_session_create(community, peername, remote_port, 
                                  converse, checkp);
         if (!sp) {
-                radlog(L_ERR,
-                       _("can't create snmp session: %s"),
+                grad_log(L_ERR,
+                         _("can't create snmp session: %s"),
                          snmp_strerror(snmp_errno));
-                efree(snmp_oid);
                 snmp_free(oid);
                 return -1;
         }
         
         if (snmp_session_open(sp, myip, 0, timeout, retries)) {
-                radlog(L_ERR,
-                       _("can't open snmp session: %s"),
+                grad_log(L_ERR,
+                         _("can't open snmp session: %s"),
                          snmp_strerror(snmp_errno));
-                efree(snmp_oid);                
                 snmp_free(oid);
                 return -1;
         }
 
         if ((pdu = snmp_pdu_create(SNMP_PDU_GET)) == NULL) {
-                radlog(L_ERR,
-                       _("can't create SNMP PDU: %s"),
+                grad_log(L_ERR,
+                         _("can't create SNMP PDU: %s"),
                          snmp_strerror(snmp_errno));
-                efree(snmp_oid);
                 snmp_free(oid);
                 snmp_session_close(sp);
                 return -1;
         }
                 
         if ((var = snmp_var_create(oid)) == NULL) {
-                radlog(L_ERR,
-                       _("can't create SNMP PDU: %s"),
+                grad_log(L_ERR,
+                         _("can't create SNMP PDU: %s"),
                          snmp_strerror(snmp_errno));
-                efree(snmp_oid);
                 snmp_free(oid);
                 snmp_session_close(sp);
                 snmp_pdu_free(pdu);
@@ -310,8 +361,6 @@ snmp_check(struct check_instance *checkp, NAS *nas)
                   community,
                   snmp_oid));
 
-        efree(snmp_oid);
-
         checkp->result = rc;
         
         snmp_query(sp, pdu);
@@ -335,7 +384,7 @@ alrm_handler()
 #define MIN(a,b) ((a)<(b))?(a):(b)
 
 int
-finger_check(struct check_instance *checkp, NAS *nas)
+finger_check(struct check_instance *checkp, grad_nas_t *nas)
 {
         char *arg;
         char namebuf[RUT_NAMESIZE+1];
@@ -350,23 +399,24 @@ finger_check(struct check_instance *checkp, NAS *nas)
         struct msghdr msg;
         int found = 0;
         char *peername;
-        struct obstack stk;
         char *ptr;
         RETSIGTYPE (*handler)() = SIG_IGN;
         unsigned int to;
-
+	
         arg = checkrad_xlat(checkp, slookup(checkp, "arg", "%u"));
+	if (!arg) /* The diagnostics has already been issued */
+		return -1;
+	
         /* Copy at most RUT_NAMESIZE bytes from the user name */
         ptr = arg;
         for (i = 0; i < RUT_NAMESIZE && *ptr; ++ptr, ++i)
                 namebuf[i] = *ptr;
         namebuf[i] = 0;
         namelen = i;
-        efree(arg);
         
         peername = slookup(checkp, "host", nas->longname);
         if (!(hp = gethostbyname(peername))) {
-                radlog(L_ERR, _("unknown host: %s"), peername);
+                grad_log(L_ERR, _("unknown host: %s"), peername);
                 return -1;
         }
 
@@ -385,7 +435,7 @@ finger_check(struct check_instance *checkp, NAS *nas)
                MIN(hp->h_length,sizeof(sin.sin_addr)));
         sin.sin_port = port;
         if ((s = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0) {
-                radlog(L_ERR|L_PERROR, "socket");
+                grad_log(L_ERR|L_PERROR, "socket");
                 return -1;
         }
 
@@ -410,18 +460,16 @@ finger_check(struct check_instance *checkp, NAS *nas)
          */
         if (ilookup(checkp, "tcp", 1) &&
             connect(s, (struct sockaddr *)&sin, sizeof (sin))) {
-                radlog(L_ERR|L_PERROR, "connect");
+                grad_log(L_ERR|L_PERROR, "connect");
                 return -1;
         }
 
         if (sendmsg(s, &msg, 0) < 0) {
-                radlog(L_ERR|L_PERROR, "sendmsg");
+                grad_log(L_ERR|L_PERROR, "sendmsg");
                 close(s);
                 return -1;
         }
 
-        obstack_init(&stk);
-
         /*
          * Read from the remote system; If no data arrives, we will exit
          * by alarm.
@@ -431,18 +479,17 @@ finger_check(struct check_instance *checkp, NAS *nas)
         lastc = 0;
         if ((fp = fdopen(s, "r")) != NULL) {
 		if (setjmp(to_env)) {
-			radlog(L_NOTICE,
+			grad_log(L_NOTICE,
 			       _("timed out in waiting for finger response from NAS %s"),
-			       checkp->hostname);
+			       checkp->nasname);
 			fclose(fp);
-			obstack_free(&stk, NULL);
 			alarm(0);
-			signal(SIGALRM, handler);
+			grad_set_signal(SIGALRM, handler);
 			return checkp->result = -1;
 		}
 
                 to = ilookup(checkp, "timeout", 10);
-		handler = signal(SIGALRM, alrm_handler);
+		handler = grad_set_signal(SIGALRM, alrm_handler);
 		alarm(to);
 
                 while ((c = getc(fp)) != EOF) {
@@ -463,24 +510,24 @@ finger_check(struct check_instance *checkp, NAS *nas)
                                         continue;
                                 }
                         }
-                        obstack_1grow(&stk, c);
+                        obstack_1grow(&checkp->stack, c);
                         if (c == '\n') {
                                 /* Make sure no alarm arrives while
                                  * processing data
                                  */
 				to = alarm(0);
-				signal(SIGALRM, handler);
+				grad_set_signal(SIGALRM, handler);
                                 
-                                obstack_1grow(&stk, 0);
-                                ptr = obstack_finish(&stk);
+                                obstack_1grow(&checkp->stack, 0);
+                                ptr = obstack_finish(&checkp->stack);
                                 debug(2,("got : %s", ptr));
                                 found = compare(checkp, ptr);
-                                obstack_free(&stk, ptr);
+                                obstack_free(&checkp->stack, ptr);
                                 if (found) 
                                         break;
 
                                 /* restore alarm settings */
-				signal(SIGALRM, alrm_handler);
+				grad_set_signal(SIGALRM, alrm_handler);
 				alarm(to);
                         }
                 }
@@ -490,25 +537,23 @@ finger_check(struct check_instance *checkp, NAS *nas)
                          * processing data
                          */
                         alarm(0);
-			signal(SIGALRM, handler);
+			grad_set_signal(SIGALRM, handler);
 
-                        obstack_1grow(&stk, '\n');
-                        obstack_1grow(&stk, 0);
+                        obstack_1grow(&checkp->stack, '\n');
+                        obstack_1grow(&checkp->stack, 0);
                         debug(2,("got : %s", ptr));
-                        ptr = obstack_finish(&stk);
+                        ptr = obstack_finish(&checkp->stack);
                         found = compare(checkp, ptr);
                 }
-                obstack_free(&stk, NULL);
                 
                 if (ferror(fp)) {
-                        radlog(L_ERR|L_PERROR, "finger");
+                        grad_log(L_ERR|L_PERROR, "finger");
                 }
                 fclose(fp);
         }
 
         /* restore alarm settings */
         alarm(0);
-	signal(SIGALRM, handler);
 
         debug(1, ("result: %d", found));
         checkp->result = found;
@@ -517,14 +562,59 @@ finger_check(struct check_instance *checkp, NAS *nas)
 
 /*ARGSUSED*/
 int
-ext_check(struct check_instance *checkp, NAS *nas)
+ext_check(struct check_instance *checkp, grad_nas_t *nas)
 {
-        radlog(L_ERR, "ext_check not implemented");
+	char *s, *path;
+	int rc;
+	
+	s = slookup(checkp, "path", NULL);
+	if (!s) {
+                grad_log(L_ERR, _("path variable not set"));
+                return -1;
+	}
+	path = checkrad_xlat_new(checkp, s);
+	if (!path) /* The diagnostics has already been issued */
+		return -1;
+
+	rc = radius_exec_command(path);
+	switch (rc) {
+	case 0:
+	case 1:
+		return rc;
+	default:
+		return -1;
+	}
         return -1;
 }
 
 int
-checkrad(NAS *nas, struct radutmp *up)
+guile_check(struct check_instance *checkp, grad_nas_t *nas)
+{
+	char *s, *expr;
+	int rc;
+	
+	s = slookup(checkp, "expr", NULL);
+	if (!s) {
+                grad_log(L_ERR, _("expr variable not set"));
+                return -1;
+	}
+	expr = checkrad_xlat_new(checkp, s);
+	if (!expr) /* The diagnostics has already been issued */
+		return -1;
+
+	rc = scheme_eval_boolean_expr(expr);
+	switch (rc) {
+	case 0:
+	case 1:
+		return rc;
+	default:
+		return -1;
+	}
+        return -1;
+}
+
+int
+checkrad(grad_nas_t *nas, struct radutmp *up)
 {
         struct check_instance checkp;
         int rc = -1;
@@ -542,8 +632,11 @@ checkrad(NAS *nas, struct radutmp *up)
         case METHOD_EXT:
                 rc = ext_check(&checkp, nas);
                 break;
+	case METHOD_GUILE:
+		rc = guile_check(&checkp, nas);
+		break;
         default:
-                insist_fail("bad method");
+                grad_insist_fail("bad method");
         }
         free_instance(&checkp);
         return rc;
