@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2001,2003 Free Software Foundation, Inc.
+   Copyright (C) 2001,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
   
@@ -21,9 +21,8 @@
 # include <config.h>
 #endif
 #include <limits.h>
-#include <symtab.h>
+
 #include <radiusd.h>
-#include <parser.h>
 #include <rewrite.h>
 
 #ifndef CHAR_BIT
@@ -38,51 +37,50 @@
 #define BITISSET(x, i)  (((x)[(i)/BITS_PER_WORD] & (1<<((i) % BITS_PER_WORD))) != 0)
 
 struct check_datum {
-        Symtab   *symtab;
+        grad_symtab_t   *symtab;
         unsigned count;    /* Number of elements */
         unsigned rlen;     /* Number of elements in a row */
         unsigned *r;
 };
 
 static void TC(unsigned *R, int n);
-static int sym_counter(void *closure, User_symbol *sym);
 static void mark_profile(struct check_datum *datum, User_symbol *sym,
                          char *target_name);
 static void mark_list(struct check_datum *datum, User_symbol *sym, 
-                      VALUE_PAIR *list);
-static int pass1(struct check_datum *datum, User_symbol *sym);
-static int pass2(struct check_datum *datum, User_symbol *sym);
-static void check_dup_attr(VALUE_PAIR **prev, VALUE_PAIR *ptr, LOCUS *loc);
+                      grad_avp_t *list);
+static void check_dup_attr(grad_avp_t **prev, grad_avp_t *ptr, grad_locus_t *loc);
 
 
-int
-sym_counter(void *closure, User_symbol *sym)
+static int
+sym_counter(void *closure, grad_symbol_t *symbol)
 {
+	User_symbol *sym = (User_symbol *) symbol;
+
         sym->ordnum = (*(int*)closure)++;
         return 0;
 }
 
-void
+static void
 radck_setbit(unsigned *r, unsigned rowsize, unsigned row, unsigned col)
 {
         SETBIT(r + rowsize * row, col);
 }
 
-int
+static int
 radck_bitisset(unsigned *r, unsigned rowsize, unsigned row, unsigned col)
 {
         return BITISSET(r + rowsize * row, col);
 }
 
-void
+static void
 mark_profile(struct check_datum *datum, User_symbol *sym, char *target_name)
 {
-        User_symbol *target = (User_symbol*)sym_lookup(datum->symtab, target_name);
+        User_symbol *target = (User_symbol*)grad_sym_lookup(datum->symtab, target_name);
 
         if (!target) {
-                radlog_loc(L_ERR, &sym->loc,
-			   _("Match-Profile refers to non-existing profile (%s)"),
-			   target_name);
+                grad_log_loc(L_ERR, &sym->loc,
+			     _("Match-Profile refers to non-existing profile (%s)"),
+			     target_name);
                 return;
         }
         
@@ -93,35 +91,41 @@ mark_profile(struct check_datum *datum, User_symbol *sym, char *target_name)
                  !strcmp(target->name, target_name));
 }
 
-void
-mark_list(struct check_datum *datum, User_symbol *sym, VALUE_PAIR *list)
+static void
+mark_list(struct check_datum *datum, User_symbol *sym, grad_avp_t *list)
 {
-        VALUE_PAIR *p;
+        grad_avp_t *p;
 
-        if (p = avl_find(list, DA_MATCH_PROFILE)) {
+        if (p = grad_avl_find(list, DA_MATCH_PROFILE)) {
                 do {
                         mark_profile(datum, sym, p->avp_strvalue);
                 } while (p->next &&
-                         (p = avl_find(p->next, DA_MATCH_PROFILE)));
+                         (p = grad_avl_find(p->next, DA_MATCH_PROFILE)));
         }
 }
 
-int
-pass1(struct check_datum *datum, User_symbol *sym)
+static int
+pass1(void *closure, grad_symbol_t *symbol)
 {
+	struct check_datum *datum = closure;
+	User_symbol *sym = (User_symbol *) symbol;
+
         mark_list(datum, sym, sym->check);
         mark_list(datum, sym, sym->reply);
         return 0;
 }
 
-int
-pass2(struct check_datum *datum, User_symbol *sym)
+static int
+pass2(void *closure, grad_symbol_t *symbol)
 {
+	struct check_datum *datum = closure;
+	User_symbol *sym = (User_symbol *) symbol;
+
         if (radck_bitisset(datum->r, datum->rlen, sym->ordnum, sym->ordnum)) {
-                radlog_loc(L_ERR, &sym->loc,
-			   _("circular dependency for %s"),
-			   sym->name);
-                symtab_delete(datum->symtab, (Symbol *)sym);
+                grad_log_loc(L_ERR, &sym->loc,
+			     _("circular dependency for %s"),
+			     sym->name);
+                grad_symtab_delete(datum->symtab, (grad_symbol_t *)sym);
                 datum->count--;
         }
         return 0;
@@ -138,15 +142,15 @@ radck()
          * Count users.
          */
         user_count = 0;
-        symtab_iterate(user_tab, sym_counter, &user_count);
+        grad_symtab_iterate(user_tab, sym_counter, &user_count);
 
         if (user_count) {
 		/* Allocate matrix */
 		size = (user_count + BITS_PER_WORD - 1) / BITS_PER_WORD;
-		r = radxmalloc(user_count*size*sizeof(unsigned));
+		r = grad_malloc(user_count*size*sizeof(unsigned));
 		if (!r) {
-			radlog(L_ERR,
-			       _("not enough memory for transitivity check"));
+			grad_log(L_ERR,
+			         _("not enough memory for transitivity check"));
 			return;
 		}
 
@@ -157,42 +161,42 @@ radck()
 		datum.r      = r;
 		
 		/* First pass: mark directly connected entries */
-		symtab_iterate(user_tab, pass1, &datum);
+		grad_symtab_iterate(user_tab, pass1, &datum);
 
 		/* Compute transitive closure of the matrix r */
 		TC(datum.r, user_count);
 
 		/* Select all non-zero diagonal elements and delete
 		   corresponding profiles  */
-		symtab_iterate(user_tab, pass2, &datum);
-		efree(datum.r);
+		grad_symtab_iterate(user_tab, pass2, &datum);
+		grad_free(datum.r);
 
 		user_count = datum.count;
 	}
 	
         if (user_count == 0) 
-                radlog(L_ERR, _("USER LIST IS EMPTY"));
+                grad_log(L_ERR, _("USER LIST IS EMPTY"));
 }
 
 static void
-check_dup_attr(VALUE_PAIR **prev, VALUE_PAIR *ptr, LOCUS *loc)
+check_dup_attr(grad_avp_t **prev, grad_avp_t *ptr, grad_locus_t *loc)
 {
         if (*prev) {
-                radlog_loc(L_WARN, loc,
-			   _("duplicate %s attribute"), ptr->name);
+                grad_log_loc(L_WARN, loc,
+			     _("duplicate %s attribute"), ptr->name);
         } else
                 *prev = ptr;
 }
 
 static int
-compile_pair(VALUE_PAIR *pair)
+compile_pair(grad_avp_t *pair)
 {
-	if (pair->eval_type == eval_interpret) {
+	if (pair->eval_type == grad_eval_interpret) {
 		char *symname = rewrite_compile(pair->avp_strvalue);
 		if (symname == 0) 
 			return -1;
-		pair->eval_type = eval_compiled;
-		efree(pair->avp_strvalue);
+		pair->eval_type = grad_eval_compiled;
+		grad_free(pair->avp_strvalue);
 		pair->avp_strvalue = symname;
 		pair->avp_strlength = strlen(symname);
 	}
@@ -201,27 +205,27 @@ compile_pair(VALUE_PAIR *pair)
 
 /*ARGSUSED*/
 int
-fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
+fix_check_pairs(int cf_file, grad_locus_t *loc, char *name, grad_avp_t **pairs)
 {
-        VALUE_PAIR *p;
-        VALUE_PAIR *auth_type = NULL;
-        VALUE_PAIR *auth_data = NULL;
-        VALUE_PAIR *pam_auth = NULL;
-        VALUE_PAIR *password = NULL;
-        VALUE_PAIR *crypt_password = NULL;
-        VALUE_PAIR *chap_password = NULL;
-        VALUE_PAIR *pass_loc = NULL;
-        DICT_ATTR *dict;
+        grad_avp_t *p;
+        grad_avp_t *auth_type = NULL;
+        grad_avp_t *auth_data = NULL;
+        grad_avp_t *pam_auth = NULL;
+        grad_avp_t *password = NULL;
+        grad_avp_t *crypt_password = NULL;
+        grad_avp_t *chap_password = NULL;
+        grad_avp_t *pass_loc = NULL;
+        grad_dict_attr_t *dict;
         int errcnt = 0;
         
         for (p = *pairs; p; p = p->next) {
                 
-                dict = attr_number_to_dict(p->attribute);
+                dict = grad_attr_number_to_dict(p->attribute);
                 if (dict) {
-                        if (!(dict->prop & AF_LHS(cf_file))) {
-                                radlog_loc(L_ERR, loc,
+                        if (!(dict->prop & GRAD_AF_LHS(cf_file))) {
+                                grad_log_loc(L_ERR, loc,
 					 _("attribute %s not allowed in LHS"),
-					   dict->name);
+					     dict->name);
                                 errcnt++;
                                 continue;
                         }
@@ -250,6 +254,13 @@ fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
                         break;
                         
                 case DA_PASSWORD_LOCATION:
+			if (p->avp_lvalue == DV_PASSWORD_LOCATION_SQL) {
+				const char *msg;
+				if (!sql_auth_avail_p(&msg)) {
+					grad_log_loc(L_ERR, loc, "%s", msg);
+					errcnt++;
+				}
+			}
                         check_dup_attr(&pass_loc, p, loc);
                         break;
 
@@ -260,16 +271,24 @@ fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
                 case DA_MATCH_PROFILE:
                         if (strncmp(p->avp_strvalue, "DEFAULT", 7) == 0 ||
                             strncmp(p->avp_strvalue, "BEGIN", 5) == 0) {
-                                radlog_loc(L_ERR, loc,
-					   "%s",
-				       _("Match-Profile refers to a DEFAULT entry"));
+                                grad_log_loc(L_ERR, loc,
+					     "%s",
+				  _("Match-Profile refers to a DEFAULT entry"));
                                 errcnt++;
                         }
                         break;
+
+		case DA_SIMULTANEOUS_USE:
+			if (!radius_mlc_enabled_p()) {
+				grad_log_loc(L_ERR, loc,
+					     "%s",
+					     _("Simultaneous-Use is used, but multiple login checking is not enabled"));
+				errcnt++;
+			}
                 }
         }
 
-        if (cf_file != CF_USERS)
+        if (cf_file != GRAD_CF_USERS)
                 return 0;
 
         /*
@@ -288,30 +307,30 @@ fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
                                 type = DV_AUTH_TYPE_PAM;
                         else if (!strcmp(password->avp_strvalue, "MYSQL")
                                  || !strcmp(password->avp_strvalue, "SQL"))
-                                type = DV_AUTH_TYPE_MYSQL;
+                                type = DV_AUTH_TYPE_SQL;
                         else
                                 type = DV_AUTH_TYPE_LOCAL;
                 } else {
                         return 0;
                 }
-                auth_type = avp_create_integer(DA_AUTH_TYPE, type);
-                avl_add_pair(pairs, auth_type);
+                auth_type = grad_avp_create_integer(DA_AUTH_TYPE, type);
+                grad_avl_add_pair(pairs, auth_type);
         }
         
         switch (auth_type->avp_lvalue) {
         case DV_AUTH_TYPE_LOCAL:
                 if (!password && !chap_password && !pass_loc) {
-                        radlog_loc(L_ERR, loc,
-				   "%s",
-				   _("No User-Password attribute in LHS"));
+                        grad_log_loc(L_ERR, loc,
+				     "%s",
+				     _("No User-Password attribute in LHS"));
                         errcnt++;
                 }
                 break;
                 
         case DV_AUTH_TYPE_SYSTEM:
 		if (radiusd_user.username) {
-			radlog_loc(L_ERR, loc,
-				   _("Auth-Type = System can only be used when running with root privileges"));
+			grad_log_loc(L_ERR, loc,
+				     _("Auth-Type = System can only be used when running with root privileges"));
 			errcnt++;
 		}
 		/*FALLTHROUGH*/
@@ -319,55 +338,64 @@ fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
         case DV_AUTH_TYPE_REJECT:
         case DV_AUTH_TYPE_ACCEPT:
                 if (password) {
-                        radlog_loc(L_WARN, loc,
-				   "%s",
+                        grad_log_loc(L_WARN, loc,
+				     "%s",
 			       _("User-Password attribute ignored for this Auth-Type"));
                 }
                 if (pass_loc) {
-                        radlog_loc(L_WARN, loc,
-				   "%s",
+                        grad_log_loc(L_WARN, loc,
+				     "%s",
 			       _("Password-Location attribute ignored for this Auth-Type"));
                 }
                 break;
                 
         case DV_AUTH_TYPE_CRYPT_LOCAL:
                 if (!password && !crypt_password && !pass_loc) {
-                        radlog_loc(L_ERR, loc,
-				   "%s",
+                        grad_log_loc(L_ERR, loc,
+				     "%s",
 			       _("No User-Password attribute in LHS"));
                         errcnt++;
                 }
                 break;
 
         case DV_AUTH_TYPE_SECURID:
-                radlog_loc(L_ERR, loc,
-			   "%s",
+                grad_log_loc(L_ERR, loc,
+			     "%s",
 			   _("Authentication type not supported"));
                 errcnt++;
                 break;
                 
         case DV_AUTH_TYPE_SQL:
+	{
+		const char *msg;
+		if (!sql_auth_avail_p(&msg)) {
+			grad_log_loc(L_ERR, loc, "%s", msg);
+			errcnt++;
+			break;
+		}
+		
                 if (password || crypt_password) {
-                        radlog_loc(L_WARN, loc,
-				   "%s",
+                        grad_log_loc(L_WARN, loc,
+				     "%s",
 			       _("User-Password attribute ignored for this Auth-Type"));
                 }
 
-                avl_delete(pairs, DA_AUTH_TYPE);
-                p = avp_create_integer(DA_AUTH_TYPE, 
-                                       DV_AUTH_TYPE_CRYPT_LOCAL);
-                avl_add_pair(pairs, p);
+                grad_avl_delete(pairs, DA_AUTH_TYPE);
+                p = grad_avp_create_integer(DA_AUTH_TYPE, 
+					    DV_AUTH_TYPE_CRYPT_LOCAL);
+                grad_avl_add_pair(pairs, p);
                 
-                p = avp_create_integer(DA_PASSWORD_LOCATION, 
-                                       DV_PASSWORD_LOCATION_SQL);
-                avl_add_pair(pairs, p);
+                p = grad_avp_create_integer(DA_PASSWORD_LOCATION, 
+					    DV_PASSWORD_LOCATION_SQL);
+                grad_avl_add_pair(pairs, p);
                 
                 break;
-                
+	}
+	
         case DV_AUTH_TYPE_PAM:
                 if (pam_auth && auth_data) {
-                        radlog_loc(L_WARN, loc,
-			       "%s",
+                        grad_log_loc(L_WARN, loc,
+			             "%s",
 			       _("Both Auth-Data and PAM-Auth attributes present"));
                         auth_data = NULL;
                 } else 
@@ -379,20 +407,20 @@ fix_check_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
 }
 
 int
-fix_reply_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
+fix_reply_pairs(int cf_file, grad_locus_t *loc, char *name, grad_avp_t **pairs)
 {
-        VALUE_PAIR *p;
+        grad_avp_t *p;
         int fall_through = 0;
-        DICT_ATTR *dict;
+        grad_dict_attr_t *dict;
         int errcnt = 0;
         
         for (p = *pairs; p; p = p->next) {
-                dict = attr_number_to_dict(p->attribute);
+                dict = grad_attr_number_to_dict(p->attribute);
                 if (dict) {
-                        if (!(dict->prop & AF_RHS(cf_file))) {
-                                radlog_loc(L_ERR, loc,
+                        if (!(dict->prop & GRAD_AF_RHS(cf_file))) {
+                                grad_log_loc(L_ERR, loc,
 					  _("attribute %s not allowed in RHS"),
-					   dict->name);
+					     dict->name);
                                 errcnt++;
                                 continue;
                         }
@@ -405,7 +433,7 @@ fix_reply_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
                         break;
 
 		case DA_ADD_PORT_TO_IP_ADDRESS:
-			radlog_loc(L_ERR, loc,
+			grad_log_loc(L_ERR, loc,
 			     _("Use of Add-Port-To-IP-Address is deprecated"));
 			errcnt++;
                 }
@@ -415,7 +443,7 @@ fix_reply_pairs(int cf_file, LOCUS *loc, char *name, VALUE_PAIR **pairs)
         }
 
         if (strncmp(name, "BEGIN", 5) == 0 && fall_through == 0) {
-                radlog_loc(L_WARN, loc, "%s", _("BEGIN without Fall-Through"));
+                grad_log_loc(L_WARN, loc, "%s", _("BEGIN without Fall-Through"));
         }
         return errcnt;
 }
