@@ -17,7 +17,7 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-/* RPP is a Radius Process Pool */
+/* RPP stands for Radius Process Pool */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -29,53 +29,111 @@
 
 #include <radiusd.h>
 
+static int rpp_stdin;
+static int rpp_stdout;
+
 /* Process intercommunication primitives */
 
+enum process_status {
+	process_busy,      /* Process is busy handling a request */
+	process_ready,     /* Precess is idle and ready for input */
+	process_finished   /* Process has finished */
+};
+
 typedef struct {
-	pid_t pid;   
-	int p[2];
-	int ready;
+	pid_t pid;    /* PID of the handler process */
+	int p[2];     /* IPC descriptors. p[0] - input, p[1] - output */
+	enum process_status status; /* Current process status */
+	int exit_status;  /* Process exit status if status==process_finished */
 } rpp_proc_t;
 
+
+/* Low-level calls */
+
+/* Recompute the timeout TVAL taking into account the time elapsed since
+   START */
+static int
+recompute_timeout(struct timeval *start, struct timeval *tval)
+{
+	struct timeval now, diff;
+	
+	gettimeofday(&now, NULL);
+	timersub(&now, start, &diff);
+	if (timercmp(&diff, tval, <)) {
+		struct timeval tmp;
+		timersub(tval, &diff, &tmp);
+		*tval = tmp;
+		return 0;
+	} 
+	return 1;
+}
+
+/* Write into the pipe (represented by FD) SIZE bytes from PTR. TV
+   sets timeout. TV==NULL means no timeout is imposed.
+
+   Returns number of bytes written */
 static int
 pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 {
+	errno = 0;
 	if (!tv)
 		return write(fd, ptr, size);
 	else {
 		char *data = ptr;
 		int rc;
-		struct timeval to;
+		struct timeval tval, start;
 		fd_set wr_set;
 		size_t n;
-		
+
+		gettimeofday(&start, NULL);
 		for (n = 0; n < size;) {
-			to = *tv;
+
 			FD_ZERO(&wr_set);
 			FD_SET(fd, &wr_set);
-			rc = select(fd + 1, NULL, &wr_set, NULL, &to);
-			if (rc == 0)
+
+			tval = *tv;
+			if (recompute_timeout (&start, &tval)) {
+				errno = ETIMEDOUT;
 				break;
-			else if (rc < 0) {
+			}
+			
+			rc = select(fd + 1, NULL, &wr_set, NULL, &tval);
+			if (rc == 0) {
+				errno = ETIMEDOUT;
+				debug(100, ("rc = 0"));
+				break;
+			} else if (rc < 0) {
+				if (errno == EINTR) 
+					continue;
+				debug(100, ("rc = %d, errno = %d", rc, errno));
 				break;
 			} else if (rc > 0) {
 				rc = write(fd, data, 1);
-				if (rc != 1) 
+				if (rc != 1) {
+					debug(100, ("rc = %d, errno = %d", rc,
+                                              errno));
 					break;
+				}
 				data++;
 				n++;
 			}
 		}
+		debug(100,("n = %d",n));
 		return n;
 	}
 }
 
+/* Read from the pipe (represented by FD) at most SIZE bytes into PTR. TV
+   sets timeout. TV==NULL means no timeout is imposed.
+
+   Returns number of bytes read */
 static int
 pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 {
 	char *data = ptr;
 	int rc;
 
+	errno = 0;
 	if (!tv) {
 		int rdbytes = 0;
 		do {
@@ -84,23 +142,35 @@ pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 				data += rc;
 				size -= rc;
 				rdbytes += rc;
-			} else 
+			} else if (errno != EINTR)
 				break;
 		} while (size > 0);
 		return rdbytes;
 	} else {
-		struct timeval to;
+		struct timeval tval, start;
 		fd_set rd_set;
 		size_t n;
 		
+		gettimeofday(&start, NULL);
 		for (n = 0; n < size;) {
-			to = *tv;
+
 			FD_ZERO(&rd_set);
 			FD_SET(fd, &rd_set);
-			rc = select(fd + 1, &rd_set, NULL, NULL, &to);
-			if (rc == 0)
+
+			tval = *tv;
+			if (recompute_timeout (&start, &tval)) {
+				errno = ETIMEDOUT;
 				break;
+			}
+
+			rc = select(fd + 1, &rd_set, NULL, NULL, &tval);
+			if (rc == 0) {
+				errno = ETIMEDOUT;
+				break;
+			}
 			if (rc < 0) {
+				if (errno == EINTR)
+					continue;
 				break;
 			} else if (rc > 0) {
 				rc = read(fd, data, 1);
@@ -114,14 +184,20 @@ pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 	}
 }
 
+
+/* RPP layer I/O. Each packet transmitted over a pipe is preceeded by
+   its length (a size_t number in host order) */
+
+/* Read SIZE bytes from the pipe (FD) into DATA. TV sets timeout */
 static int
 rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 {
-	size_t sz, nbytes;
+	size_t sz, nbytes = 0;
 
 	sz = pipe_read(fd, &nbytes, sizeof(nbytes), tv);
 	if (sz == 0)
 		return 0; /* eof */
+	debug(100,("nbytes=%lu",nbytes));
 	if (sz != sizeof(nbytes)) 
 		return -1;
 	sz = nbytes > size ? size : nbytes;
@@ -129,22 +205,34 @@ rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 		return -2;
 	for (;nbytes > size; nbytes--) {
 		char c;
-		pipe_read(fd, &c, 1, tv);
+		if (pipe_read(fd, &c, 1, tv) != 1) 
+			return -3;
 	}
 	
+	debug(100,("return %lu", (unsigned long)sz));
 	return sz;
 }
 
+/* Write SIZE bytes from DATA to the pipe FD. TV sets timeout */
 static int
 rpp_fd_write(int fd, void *data, size_t size, struct timeval *tv)
 {
-	if (pipe_write(fd, &size, sizeof(size), tv) != sizeof(size))
+	int rc;
+	debug(100,("size=%lu",size));
+	if (pipe_write(fd, &size, sizeof(size), tv) != sizeof(size)) 
 		return -1;
-	if (pipe_write(fd, data, size, tv) != size)
+	if (pipe_write(fd, data, size, tv) != size) 
 		return -2;
+	debug(1,("return %lu", (unsigned long)size));
 	return size;
 }
 
+
+
+/* Start a handler process. PROC_MAIN is the handler function (process'
+   main loop). DATA is passed to PROC_MAIN verbatim.
+
+   On success return 0 and fill in PROC structure. */
 int
 rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 {
@@ -169,18 +257,17 @@ rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 	}
 	if (pid == 0) {
 		/* Child */
-
 		/* Close remote side of pipes */
 		close(inp[0]);
 		close(outp[1]);
-		/* Close stdio */
+		/* Close stdin */
 		close(0);
-		close(1);
-		
-		/* Duplicate i/o channels to stdio */
-		dup(outp[0]);  /* This becomes standard in */
-		dup(inp[1]);   /* and this standard out */
+		/* Redirect stdout to stderr */
+		dup2(2, 1);
 
+		rpp_stdin = outp[0];
+		rpp_stdout = inp[1];
+		
 		/* Run the main process */
 		exit(proc_main(data));
 	}
@@ -192,7 +279,7 @@ rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 	proc->pid = pid;
 	proc->p[0] = inp[0];
 	proc->p[1] = outp[1];
-	proc->ready = 1;
+	proc->status = process_ready;
 	return 0;
 }
 
@@ -200,6 +287,7 @@ rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 
 static grad_list_t *process_list; /* List of rpp_proc_t */
 
+/* Look up the rpp_proc_t whose input descriptor is FD */
 rpp_proc_t *
 rpp_lookup_fd(int fd)
 {
@@ -212,6 +300,9 @@ rpp_lookup_fd(int fd)
 	return p;
 }
 
+/* Find an rpp_proc_t ready for handling the reqest. If none is found,
+   start a new one. PROC_MAIN and DATA have the same meaning as in
+   rpp_start_process() */
 rpp_proc_t *
 rpp_lookup_ready(int (*proc_main)(void *), void *data)
 {
@@ -219,7 +310,8 @@ rpp_lookup_ready(int (*proc_main)(void *), void *data)
 
 	if (process_list) {
 		grad_iterator_t *itr = grad_iterator_create(process_list);
-		for (p = grad_iterator_first(itr); p && !p->ready;
+		for (p = grad_iterator_first(itr);
+		     p && p->status != process_ready;
 		     p = grad_iterator_next(itr))
 			;
 		grad_iterator_destroy(&itr);
@@ -242,19 +334,24 @@ rpp_lookup_ready(int (*proc_main)(void *), void *data)
 	return p;
 }
 
+/* Find the rpp_proc_t with a given PID */
+static int
+rpp_comparator(void *item, void *data)
+{
+	rpp_proc_t *p = item;
+	pid_t *pid = data;
+	return p->pid != *pid;
+}
+
+/* NOTE: Do not use any memory allocation calls */
 rpp_proc_t *
 rpp_lookup_pid(pid_t pid)
 {
-	rpp_proc_t *p;
-	grad_iterator_t *itr = grad_iterator_create(process_list);
-	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
-		if (p->pid == pid)
-			break;
-	}
-	grad_iterator_destroy(&itr);
-	return p;
+	return grad_list_locate(process_list, &pid, rpp_comparator);
 }
-	
+
+/* Remove the given rpp_proc_t from the list. The underlying process
+   is supposed to have finished. No diagnostics is output. */
 static void
 _rpp_remove(rpp_proc_t *p)
 {
@@ -265,17 +362,58 @@ _rpp_remove(rpp_proc_t *p)
 		grad_free(p);
 }
 
+/* Remove the given rpp_proc_t from the list. Output diagnostics about
+   exit status of the handler */
 void
-rpp_remove(pid_t pid)
+rpp_remove(rpp_proc_t *p)
+{
+	char buffer[128];
+	format_exit_status(buffer, sizeof buffer, p->exit_status);
+	grad_log(L_NOTICE, _("child %lu %s"),
+		 (unsigned long) p->pid, buffer);
+	_rpp_remove(p);
+}
+
+/* Remove the rpp entry with given PID */
+void
+rpp_remove_pid(pid_t pid)
 {
 	rpp_proc_t *p = rpp_lookup_pid(pid);
 	if (p)
-		_rpp_remove(p);
+		rpp_remove(p);
+}
+
+/* Reflect the change of state of the rpp handler process.
+
+   The function is safe to be used from signal handlers */
+void
+rpp_status_changed(pid_t pid, int exit_status)
+{
+	rpp_proc_t *p = rpp_lookup_pid(pid);
+	if (p) {
+		p->status = process_finished;
+		p->exit_status = exit_status;
+	}
+}
+
+/* Collect rpp_proc's whose handler have exited. Free any associated
+   resources */
+void
+rpp_collect_exited()
+{
+	rpp_proc_t *p;
+	grad_iterator_t *itr = grad_iterator_create(process_list);
+	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
+		if (p->status == process_finished)
+			rpp_remove(p);
+	}
+	grad_iterator_destroy(&itr);
 }
 
 
 static int rpp_request_handler(void *arg);
 
+/* Return 1 if rpp with the given PID is ready for input */
 int
 rpp_ready(pid_t pid)
 {
@@ -292,12 +430,14 @@ rpp_ready(pid_t pid)
 			}
 		}
 	        grad_iterator_destroy(&itr);
-		if (p && p->ready)
+		if (p && p->status == process_ready)
 			return 1;
 	}
 	return 0;
 }
 
+/* Traverse the rpp list until FUN returns non-zero or there are no
+   more busy handlers. While traversing, remove finished handlers. */
 void
 rpp_flush(int (*fun)(void*), void *closure)
 {
@@ -312,8 +452,17 @@ rpp_flush(int (*fun)(void*), void *closure)
 		for (count = 0, p = grad_iterator_first(itr);
 		     p;
 		     p = grad_iterator_next(itr))
-			if (!p->ready)
+			switch (p->status) {
+			case process_ready:
+				break;
+
+			case process_busy:
 				count++;
+				break;
+
+			case process_finished:
+				rpp_remove (p);
+			}
 	} while (count > 0 && (*fun)(closure) == 0);
 	grad_iterator_destroy(&itr);
 }
@@ -325,7 +474,8 @@ _kill_itr(void *item, void *data)
 	kill(p->pid, *(int*)data);
 	return 0;
 }
-	
+
+/* Deliver signal SIGNO to the handler with pid PID */
 int
 rpp_kill(pid_t pid, int signo)
 {
@@ -333,7 +483,7 @@ rpp_kill(pid_t pid, int signo)
 		rpp_proc_t *p = rpp_lookup_pid(pid);
 		if (p) {
 			kill(p->pid, signo);
-			_rpp_remove(p);
+			rpp_check_pid(p->pid);
 		} else
      		        return 1;
 	} else 
@@ -341,20 +491,42 @@ rpp_kill(pid_t pid, int signo)
 	return 0;
 }
 
-static void
-_rpp_slay(rpp_proc_t *p)
+pid_t
+rpp_check_pid(pid_t pid)
 {
-	grad_log(L_NOTICE, _("Killing unresponding process %lu"), (u_long) p->pid);
+	int status;
+	rpp_proc_t *p = rpp_lookup_pid(pid);
+	if (!p)
+		return -1;
+	if (p->status != process_finished) {
+		pid_t npid = waitpid(pid, &status, WNOHANG);
+		if (npid <= 0)
+			return pid;
+		rpp_status_changed(pid, status);
+	}
+	rpp_remove (p);
+	return 0;
+}
+
+/* Slay the child */
+static void
+_rpp_slay(rpp_proc_t *p, char *msg)
+{
+	grad_log(L_NOTICE, _("Killing process %lu: %s"), 
+                 (u_long) p->pid, msg);
 	kill(p->pid, SIGKILL);
 	_rpp_remove(p);
 }
 
+/* Return number of the handlers registered in the process list */
 size_t
 rpp_count()
 {
 	return grad_list_count(process_list);
 }
 
+
+
 struct rpp_request {
 	int type;                  /* Request type */
 	struct sockaddr_in addr;   /* Sender address */
@@ -394,7 +566,7 @@ rpp_forward_request(REQUEST *req)
 	frq.fd = req->fd;
 	frq.size = req->rawsize;
 	
-	p->ready = 0;
+	p->status = process_busy;
 	req->child_id = p->pid;
 
 	if (radiusd_write_timeout) {
@@ -404,9 +576,12 @@ rpp_forward_request(REQUEST *req)
 	} else
 		tvp = NULL;
 	
- 	if (rpp_fd_write(p->p[1], &frq, sizeof frq, tvp) != sizeof frq
-	    || rpp_fd_write(p->p[1], req->rawdata, req->rawsize, tvp) != req->rawsize) {
-		_rpp_slay(p);
+ 	if (rpp_fd_write(p->p[1], &frq, sizeof frq, tvp) != sizeof frq) {
+		_rpp_slay(p, _("error writing header"));
+		return 1;
+	}
+	if (rpp_fd_write(p->p[1], req->rawdata, req->rawsize, tvp) != req->rawsize) {
+		_rpp_slay(p, _("error writing data"));
 		return 1;
 	}
 	return 0;
@@ -442,6 +617,7 @@ sig_handler(int sig)
 		
 	case SIGTERM:
 	case SIGQUIT:
+		/* FIXME: Possibly unsafe, it calls free() etc. */
 	        radiusd_exit0();
 
 	case SIGCHLD:
@@ -480,7 +656,7 @@ rpp_request_handler(void *arg ARG_UNUSED)
 		int len;
 
 		alarm(process_timeout);
-		len = rpp_fd_read(0, &frq, sizeof frq, NULL);
+		len = rpp_fd_read(rpp_stdin, &frq, sizeof frq, NULL);
 		alarm(0);
 		if (len != sizeof frq) {
 			if (errno == EINTR)
@@ -496,14 +672,15 @@ rpp_request_handler(void *arg ARG_UNUSED)
 			data = grad_erealloc(data, datasize);
 		}
 		
-		if (rpp_fd_read(0, data, frq.size, NULL) != frq.size) {
+		if (rpp_fd_read(rpp_stdin, data, frq.size, NULL) != frq.size) {
 			grad_log(L_ERR,
 			         _("Child received malformed data"));
 			radiusd_exit0();
 		}
-		
+
 		req = request_create(frq.type, frq.fd, &frq.addr,
 				     data, frq.size);
+
 		req->status = RS_COMPLETED;
 		rc = request_handle(req, request_respond);
 			
@@ -511,7 +688,7 @@ rpp_request_handler(void *arg ARG_UNUSED)
 		debug(1, ("notifying the master"));
 		repl.code = RPP_COMPLETE;
 		repl.size = 0;
-		rpp_fd_write(1, &repl, sizeof repl, NULL);
+		rpp_fd_write(rpp_stdout, &repl, sizeof repl, NULL);
 		if (rc)
 			request_free(req);
 	}
@@ -526,7 +703,7 @@ rpp_input_handler(int fd, void *data)
 	rpp_proc_t *p = rpp_lookup_fd(fd);
 	struct timeval tv, *tvp;
 	int sz;
-	
+
 	grad_insist(p != NULL);
 	
 	if (radiusd_read_timeout) {
@@ -544,7 +721,7 @@ rpp_input_handler(int fd, void *data)
 			data = grad_emalloc(repl.size);
 			if (rpp_fd_read(fd, data, repl.size, tvp)
 			    != repl.size) {
-				_rpp_slay(p);
+				_rpp_slay(p, _("error reading data"));
 				grad_free(data);
 				return 1;
 			}
@@ -552,12 +729,12 @@ rpp_input_handler(int fd, void *data)
 		
 		if (p) {
 		        debug(1, ("updating pid %d", p->pid));
-			p->ready = 1;
+			p->status = process_ready;
 			request_update(p->pid, RS_COMPLETED, data);
 		} 
 		grad_free(data);
 	} else if (sz != 0) {
-		_rpp_slay(p);
+		_rpp_slay(p, _("error reading data; wrong data size returned"));
 		return 1;
 	}
 	
@@ -572,8 +749,8 @@ rpp_update(void *data, size_t size)
 
 	repl.code = RPP_UPDATE;
 	repl.size = size;
-	rpp_fd_write(1, &repl, sizeof repl, NULL);
-	rpp_fd_write(1, data, size, NULL);
+	rpp_fd_write(rpp_stdout, &repl, sizeof repl, NULL);
+	rpp_fd_write(rpp_stdout, data, size, NULL);
 	return 0;
 }
 
@@ -586,7 +763,9 @@ rpp_input_close(int fd, void *data)
 	return 0;
 }
 
+
 
+/* Don't use it. It's a debugging hook */
 int
 wd()
 {
