@@ -17,7 +17,7 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-/* $Id: version.c,v 1.33 2004/02/05 07:46:20 gray Exp $ */
+/* $Id: version.c,v 1.35 2004/09/10 17:49:40 gray Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -114,6 +114,9 @@ static char *compile_flag_str[] = {
 #endif
 #if defined(DENY_SHELL)
         "DENY_SHELL",
+#endif
+#if defined(USE_LOADABLE_MODULES)
+	"USE_LOADABLE_MODULES",
 #endif
         NULL
 };
@@ -184,3 +187,12 @@ version(FILE *stream, struct argp_state *state)
         exit(0);
 }
 
+void
+show_compilation_defaults()
+{
+	int i;
+	
+        for (i = 0; compile_flag_str[i]; i++) 
+                printf("%s\n", compile_flag_str[i]);
+}
+
