@@ -1,5 +1,7 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003, Sergey Poznyakoff
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
  
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,36 +31,13 @@
 #include <sys/stat.h>
 #include <time.h>
 
-#include <radius.h>
-#include <radargp.h>
+#include <common.h>
+#include <radius/radargp.h>
 #include <radtest.h>
-#include <argcv.h>
-
-Symtab *vartab;
-int verbose;
-extern int rad_clt_debug;
-RADIUS_SERVER_QUEUE *srv_queue;
-char *progname;
-int reply_code;
-VALUE_PAIR *reply_list;
-int debug_flag = 0;
-int abort_on_failure = 0;
-
-void init_symbols();
-static void assign(char *);
-
-int x_argmax;
-int x_argc;
-char **x_argv;
-
-int quick = 0;
-char *filename = NULL;
-char *server = NULL;
-int retry = 0;
-int timeout = 0;
+#include <radius/argcv.h>
 
 const char *argp_program_version = "radtest (" PACKAGE ") " VERSION;
-static char doc[] = N_("send arbitrary radius packets");
+static char doc[] = N_("Radius client shell");
 
 static struct argp_option options[] = {
         {NULL, 0, NULL, 0,
@@ -68,11 +47,9 @@ static struct argp_option options[] = {
         {"debug", 'x', "DEBUGSPEC", 0,
          N_("set debugging level"), 0},
         {"file", 'f', N_("FILE"), 0,
-         N_("Read input from FILE. When this option is used, all unknown"
-         " options in the form --VAR=VALUE are treated as variable"
-         " assignments"), 0},
-        {"quick", 'q', NULL, 0,
-         "FIXME: quick mode", 0},
+         N_("Read input from FILE. Stops further processing of the command line."), 0},
+        {"quick", 'q', NULL, 0,     /* FIXME */
+         "Do not read the configuration file", 0},
         {"retry", 'r', N_("NUMBER"), 0,
          N_("set number of retries"), 0},
         {"server", 's', N_("SERVER"), 0,
@@ -81,9 +58,48 @@ static struct argp_option options[] = {
          N_("set timeout"), 0},
         {"verbose", 'v', NULL, 0,
          N_("verbose mode"), 0},
+	{"no-interactive", 'i', NULL, 0,
+	 N_("disable interactive mode"), 0 },
+	{"dry-run", 'n', NULL, 0,
+	 N_("Check the input file syntax and exit"), 0 },
         {NULL, 0, NULL, 0, NULL, 0}
 };
 
+int verbose;                   /* -v (--verbose) option */
+int quick = 0;                 /* -q (--quick). A misnomer! */
+char *filename = NULL;         /* Name of the input file to execute.
+				  -f (--file) */
+/* These three are set from -s (--server) option */
+char *server = NULL;           /* Server name */
+int retry = 0;                 /* Retry count */
+int timeout = 0;               /* Timeout */
+
+int disable_readline;          /* --no-interactive */
+int dry_run;                   /* --dry-run (-n) */
+
+static void
+assign(char *s)
+{
+        char *p;
+        radtest_variable_t *var;
+        radtest_datum_t datum;
+        radtest_data_type type = rtv_undefined;
+        
+        p = strchr(s, '=');
+        if (!p) {
+                fprintf(stderr, _("assign: expected `='\n"));
+                return;
+        }
+        *p++ = 0;
+
+        type = parse_datum(p, &datum);
+        if (type == rtv_undefined)
+                return;
+        var = (radtest_variable_t*)grad_sym_install(vartab, s);
+        var->type = type;
+        var->datum = datum;
+}
+
 static error_t
 parse_opt(int key, char *arg, struct argp_state *state)
 {
@@ -106,24 +122,32 @@ parse_opt(int key, char *arg, struct argp_state *state)
 		
         case 'f':
                 filename = optarg;
-                *(int *)state->input = state->next;
-                state->next = state->argc;
+		*(int *)state->input = state->next;
+		state->next = state->argc;
                 break;
+
+	case 'i':
+		disable_readline = 1;
+		break;
+
+	case 'n':
+		dry_run = 1;
+		break;
 		
         case 't':
                 timeout = strtol(optarg, NULL, 0);
                 break;
 		
         case 'x':
-                set_debug_levels(optarg);
+                grad_set_debug_levels(optarg);
                 break;
 		
         case 'v':
                 verbose++;
-		set_module_debug_level("radpdu", 100);
-		set_module_debug_level("client", 100);
+		grad_set_module_debug_level("radpdu", 100);
+		grad_set_module_debug_level("client", 100);
                 break;
-		
+
         default:
                 return ARGP_ERR_UNKNOWN;
         }
@@ -136,55 +160,94 @@ static struct argp argp = {
         NULL,
         doc,
         rad_common_argp_child,
-        NULL, NULL
+        NULL,
+	NULL
 };
 
+int
+radtest_parse_options(int argc, char **argv)
+{
+        int index;
+	return argp_parse (&argp, argc, argv, ARGP_NO_EXIT, NULL, &index);
+}
+
+grad_server_queue_t *srv_queue; /* Server queue */
+
+grad_symtab_t *vartab;          /* Variable table */
+grad_symtab_t *functab;         /* Function table */
+ 
+int reply_code;                 /* Last radius reply code */
+grad_avp_t *reply_list;         /* Last radius reply pairs */
+
+grad_list_t *toplevel_env;      /* Top level environment */
+
+static void
+init_symbols()
+{
+        radtest_variable_t *var;
+        
+        vartab = grad_symtab_create(sizeof(radtest_variable_t), NULL);
+        var = (radtest_variable_t*) grad_sym_install(vartab, "REPLY_CODE");
+        var->type = rtv_integer;
+        var = (radtest_variable_t*) grad_sym_install(vartab, "REPLY");
+        var->type = rtv_avl;
+	var = (radtest_variable_t*) grad_sym_install(vartab, "SOURCEIP");
+	var->type = rtv_ipaddress;
+
+	functab = grad_symtab_create(sizeof(radtest_function_t), NULL);
+}
+
 int
 main(int argc, char **argv)
 {
         char *p;
-        int index;
-        
-        app_setup();
-        initlog(argv[0]);
+        int index, argind;
+	radtest_variable_t *var;
+	
+        grad_app_setup();
         init_symbols();
 
-        index = argc;
-        if (rad_argp_parse(&argp, &argc, &argv, 0, NULL, &index))
+        index = 0;
+        if (grad_argp_parse(&argp, &argc, &argv, ARGP_IN_ORDER, &argind, &index))
                 return 1;
-
+	if (index == 0)
+		index = argind;
+	
         argv += index;
         argc -= index;
 
         set_yydebug();
-        radpath_init();
-	srand(time(NULL));
+        grad_path_init();
+	srand(time(NULL)+getpid());
 	
-        if (dict_init()) {
-                radlog(L_ERR, _("error reading dictionary file"));
+        if (grad_dict_init()) {
+                grad_log(L_ERR, _("error reading dictionary file"));
                 return 1;
         }
-        srv_queue = rad_clt_create_queue(!quick, 0, 0);
+        srv_queue = grad_client_create_queue(!quick, 0, 0);
         
         if (!srv_queue)
                 return 1;
 
-        if (timeout)
+	var = (radtest_variable_t*)grad_sym_lookup(vartab, "SOURCEIP");
+	var->datum.ipaddr = srv_queue->source_ip;
+		
+	if (timeout)
                 srv_queue->timeout = timeout;
         if (retry)
                 srv_queue->retries = retry;
         if (server) {
-                RADIUS_SERVER serv;
+                grad_server_t serv;
                 int i, argc;
                 char **argv;
 
-                if (argcv_get(server, ":", &argc, &argv)) {
-                        radlog(L_ERR, "can't parse server definition");
+                if (argcv_get(server, ":", NULL, &argc, &argv)) {
+                        grad_log(L_ERR, _("can't parse server definition"));
                         exit(1);
                 }
 
                 if (argc < 3) {
-                        radlog(L_ERR, "no shared secret for the server");
+                        grad_log(L_ERR, _("no shared secret for the server"));
                         exit(1);
                 }
 
@@ -193,40 +256,44 @@ main(int argc, char **argv)
                 for (i = 0; i < argc; i++) {
                         switch (i) {
                         case 0:
-                                serv.addr = ip_gethostaddr(argv[i]);
+                                serv.addr = grad_ip_gethostaddr(argv[i]);
                                 if (!serv.addr) {
-                                        radlog(L_ERR,
-                                             "bad IP address or host name: %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad IP address or host name: %s"),
+                                                 argv[i]);
                                         exit(1);
                                 }
                                 break;
+				
                         case 2:
                                 serv.secret = argv[i];
                                 break;
+				
                         case 4:
                                 serv.port[0] = strtol(argv[i], &p, 0);
                                 if (*p) {
-                                        radlog(L_ERR,
-                                               "bad port number %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad port number %s"),
+                                                 argv[i]);
                                         break;
                                 }
                                 break;
+				
                         case 6:
                                 serv.port[1] = strtol(argv[i], &p, 0);
                                 if (*p) {
-                                        radlog(L_ERR,
-                                               "bad port number %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad port number %s"),
+                                                 argv[i]);
                                         break;
                                 }
                                 break;
+				
                         default:
                                 if (argv[i][0] != ':') {
-                                        radlog(L_ERR,
-                                               "bad separator near %s",
-                                               argv[i]);
+                                        grad_log(L_ERR,
+                                                 _("bad separator near %s"),
+                                                 argv[i]);
                                         exit(1);
                                 }
                                 break;
@@ -237,170 +304,87 @@ main(int argc, char **argv)
                         serv.port[0] = DEF_AUTH_PORT;
                 if (argc < 6)
                         serv.port[1] = DEF_ACCT_PORT;
-		rad_clt_append_server(srv_queue, rad_clt_alloc_server(&serv));
+		grad_client_append_server(srv_queue,
+					  grad_client_alloc_server(&serv));
                 argcv_free(argc, argv);
         }
 
-        if (list_count(srv_queue->servers) == 0) {
-                radlog(L_ERR,
-                       "No servers specfied. Use -s option.\n");
+        if (grad_list_count(srv_queue->servers) == 0) {
+                grad_log(L_ERR,
+                         _("No servers specfied. Use -s option.\n"));
                 exit(1);
         }
-        
-        x_argmax = argc + 2;
-        x_argv = emalloc(sizeof(x_argv[0]) * x_argmax);
-        x_argc = 0;
-        x_argv[x_argc++] = filename;
+
+	toplevel_env = grad_list_create();
+	radtest_env_add_string(toplevel_env, filename ? filename : "");
 
         for (; argc; argv++, argc--) {
-                if (argv[0][0] == '-'
-                    && argv[0][1] == '-'
-                    && (p = strchr(argv[0], '=')) != NULL
-                    && !(p > argv[0] && p[-1] == '\\')) 
-                        assign(argv[0]+2);
-                else if ((p = strchr(*argv, '=')) != NULL &&
-                         !(p > *argv && p[-1] == '\\')) 
+                if ((p = strchr(*argv, '=')) != NULL &&
+		    !(p > *argv && p[-1] == '\\')) 
                         assign(*argv);
-                else 
-                        x_argv[x_argc++] = *argv;
+                else
+			radtest_env_add_string(toplevel_env, *argv);
         }
-        x_argv[x_argc++] = NULL;
-        
-        if (open_input(filename))
-                return 1;
-        return yyparse();
+
+	return read_and_eval(filename);
 }
 
-void
-init_symbols()
-{
-        Variable *var;
-        
-        vartab = symtab_create(sizeof(Variable), NULL);
-        var = (Variable*) sym_install(vartab, "REPLY_CODE");
-        var->type = Integer;
-        var = (Variable*) sym_install(vartab, "REPLY");
-        var->type = Vector;
-}
+
 
-void
-assign(char *s)
+radtest_data_type
+parse_datum(char *p, radtest_datum_t *dp)
 {
-        char *p;
-        Variable *var;
-        union datum datum;
-        int type = Undefined;
-        
-        p = strchr(s, '=');
-        if (!p) {
-                fprintf(stderr, _("assign: expected `='\n"));
-                return;
-        }
-        *p++ = 0;
-
-        type = parse_datum(p, &datum);
-        if (type == Undefined)
-                return;
-        var = (Variable*)sym_install(vartab, s);
-        var->type = type;
-        var->datum = datum;
-}
-
-int
-parse_datum(char *p, union datum *dp)
-{
-        int type = Undefined;
+        radtest_data_type type = rtv_undefined;
         int length;
         
         if (*p == '"') {
                 length = strlen(++p);
                 if (length == 0 || p[length-1] != '"') {
                         fprintf(stderr, _("assign: missing closing quote\n"));
-                        return Undefined;
+                        return rtv_undefined;
                 }
                 p[length-1] = 0;
                 
-                type = String;
-                dp->string = estrdup(p);
+                type = rtv_string;
+                dp->string = grad_estrdup(p);
         } else if (isdigit(*p)) {
                 char *endp;
                 
                 /* This can be either an integer or an IP address */
                 dp->number = strtol(p, &endp, 0);
                 if (*endp == 0) {
-                        type = Integer;
+                        type = rtv_integer;
                 } else {
                         /* IP address */
-                        if ((dp->ipaddr = ip_gethostaddr(p)) != 0)
-                                type = Ipaddress;
+                        if ((dp->ipaddr = grad_ip_gethostaddr(p)) != 0)
+                                type = rtv_ipaddress;
                         else {
                                 fprintf(stderr, _("assign: invalid IP address: %s\n"), p);
-                                return Undefined;
+                                return rtv_undefined;
                         }
                 } 
         } else if (strchr(p, '.')) {
                 /* IP address */
-                if ((dp->ipaddr = ip_gethostaddr(p)) != 0)
-                        type = Ipaddress;
+                if ((dp->ipaddr = grad_ip_gethostaddr(p)) != 0)
+                        type = rtv_ipaddress;
                 else {
                         fprintf(stderr, _("assign: invalid IP address: %s\n"), p);
-                        return Undefined;
+                        return rtv_undefined;
                 }
         } else {
-                type = String;
-                dp->string = estrdup(p);
+                type = rtv_string;
+                dp->string = grad_estrdup(p);
         }
         return type;
 }
 
-char *
-print_ident(Variable *var)
-{
-        char buf[64];
-        switch (var->type) {
-        case Undefined:
-                return estrdup("UNDEFINED");
-                break;
-        case Integer:
-                sprintf(buf, "%d", var->datum.number);
-                return estrdup(buf);
-        case Ipaddress:
-                ip_iptostr(var->datum.ipaddr, buf);
-                return estrdup(buf);
-        case String:
-                return estrdup(var->datum.string);
-                break;
-        case Vector:
-                return estrdup("VECTOR");
-        }
-	return NULL;
-}
-
 void
-print_pairs(FILE *fp, VALUE_PAIR *pair)
+print_pairs(FILE *fp, grad_avp_t *pair)
 {
         for (; pair; pair = pair->next) {
-                fprintf(fp, " %s = ", pair->name);
-                switch (pair->type) {
-                case TYPE_STRING:
-                        fprintf(fp, "(STRING) %s", pair->avp_strvalue);
-                        break;
-
-                case TYPE_INTEGER:
-                        fprintf(fp, "(INTEGER) %ld", pair->avp_lvalue);
-                        break;
-
-                case TYPE_IPADDR:
-                        fprintf(fp, "(IP) %lx", pair->avp_lvalue);
-                        break;
-                
-                case TYPE_DATE:
-                        fprintf(fp, "(DATE) %ld", pair->avp_lvalue);
-                        break;
-                        
-                default:
-                        fprintf(fp, "(%d)", pair->type);
-                }
+		char *save;
+		fprintf(fp, " %s", grad_format_pair(pair, 1, &save));
+		free(save);
                 if (pair->next)
                         fprintf(fp, ",");
                 else
@@ -410,161 +394,170 @@ print_pairs(FILE *fp, VALUE_PAIR *pair)
 }
 
 void
-var_print(Variable *var)
+var_print(radtest_variable_t *var)
 {
-        char buf[DOTTED_QUAD_LEN];
+        char buf[GRAD_IPV4_STRING_LENGTH];
         if (!var)
                 return;
         switch (var->type) {
-        case Undefined:
+        case rtv_undefined:
                 printf("UNDEFINED");
                 break;
-        case Integer:
-                printf("%d", var->datum.number);
+		
+        case rtv_integer:
+                printf("%ld", var->datum.number);
                 break;
-        case Ipaddress:
-                printf("%s", ip_iptostr(var->datum.ipaddr, buf));
+		
+        case rtv_ipaddress:
+                printf("%s", grad_ip_iptostr(var->datum.ipaddr, buf));
                 break;
-        case String:
+
+	case rtv_bstring:
+	{
+		int len = grad_format_string_visual(NULL, 3,
+						    var->datum.bstring.ptr,
+						    var->datum.bstring.length);
+		char *pbuf = grad_emalloc(len+1);
+		grad_format_string_visual(pbuf, 3,
+					  var->datum.bstring.ptr,
+					  var->datum.bstring.length);
+		printf("%s", pbuf);
+		grad_free(pbuf);
+	}
+		break;
+		       
+        case rtv_string:
                 printf("%s", var->datum.string);
                 break;
-        case Vector:
-                printf("{");
-                print_pairs(stdout, var->datum.vector);
-                printf("}");
-                break;
-        case Builtin:
-                var->datum.builtin.print();
+		
+        case rtv_avl:
+                printf("(");
+                print_pairs(stdout, var->datum.avl);
+                printf(")");
                 break;
+
+	case rtv_pairlist:
+		abort();
         }
 }
 
 int
-var_free(Variable *var)
+var_free(radtest_variable_t *var)
 {
         switch (var->type) {
-        case String:
-                efree(var->datum.string);
+        case rtv_string:
+                grad_free(var->datum.string);
                 break;
-        case Vector:
-                avl_free(var->datum.vector);
+		
+        case rtv_avl:
+                grad_avl_free(var->datum.avl);
                 break;
+		
+	default:
+		break;
         }
+	return 0;
 }
 
 void
-tempvar_free(Variable *var)
+radtest_send(int port, int code, grad_avp_t *avl, grad_symtab_t *cntl)
 {
-        if (var->name)
-                return; /* named variables are not freed */
-	var_free(var);
-}
-
-void
-radtest_send(int port, int code, Variable *var, Symtab *cntl)
-{
-        RADIUS_REQ *auth;
-	Variable *p;
+        grad_request_t *auth;
+	radtest_variable_t *p;
 	
         if (reply_list)
-                avl_free(reply_list);
+                grad_avl_free(reply_list);
         reply_list = NULL;
         reply_code = 0;
         
-        if (var->type != Vector) {
-                parse_error(_("wrong datatype: expected vector"));
-                return;
-        }
-
 	if (!cntl) {
-		auth = rad_clt_send(srv_queue,
-				    port,
-				    code, var->datum.vector);
+		auth = grad_client_send(srv_queue, port, code, avl);
 	} else {
 		int id;
-		u_char vector[AUTH_VECTOR_LEN];
+		u_char authenticator[GRAD_AUTHENTICATOR_LENGTH];
 		int sflags = 0;
 		int retry = 1;
-		Variable *delay = (Variable*)sym_lookup(cntl, "delay");
+		radtest_variable_t *delay = (radtest_variable_t*)grad_sym_lookup(cntl, "delay");
 		
-		p = (Variable*)sym_lookup(cntl, "repeat");
+		p = (radtest_variable_t*)grad_sym_lookup(cntl, "repeat");
 		if (p)
 			retry = p->datum.number;
-		p = (Variable*)sym_lookup(cntl, "id");
+		p = (radtest_variable_t*)grad_sym_lookup(cntl, "id");
 		if (p) {
 			sflags |= RADCLT_ID;
 			id = p->datum.number;
 		}
-		p = (Variable*)sym_lookup(cntl, "keepauth");
+		p = (radtest_variable_t*)grad_sym_lookup(cntl, "keepauth");
 		if (p && p->datum.number) 
 			sflags |= RADCLT_AUTHENTICATOR;
-		auth = rad_clt_send0(srv_queue,
-				     port,
-				     code,
-				     var->datum.vector,
-				     0,
-				     &id,
-				     vector);
+		auth = grad_client_send0(srv_queue,
+					 port,
+					 code,
+					 avl,
+					 0,
+					 &id,
+					 authenticator);
 		while (--retry) {
 			if (delay)
 				sleep(delay->datum.number);
-			auth = rad_clt_send0(srv_queue,
-					     port,
-					     code,
-					     var->datum.vector,
-					     sflags,
-					     &id,
-					     vector);
+			auth = grad_client_send0(srv_queue,
+						 port,
+						 code,
+						 avl,
+						 sflags,
+						 &id,
+						 authenticator);
 		}
 	}
 
 	if (!auth)
 		return;
         reply_code = auth->code;
-        var = (Variable*)sym_lookup(vartab, "REPLY_CODE");
-        var->type = Integer;
-        var->datum.number = reply_code;
+        p = (radtest_variable_t*)grad_sym_lookup(vartab, "REPLY_CODE");
+        p->type = rtv_integer;
+        p->datum.number = reply_code;
 
-        reply_list = avl_dup(auth->request);
-        var = (Variable*)sym_lookup(vartab, "REPLY");
-        var->type = Vector;
-        var->datum.vector = NULL;
-        var->datum.vector = reply_list;
-        radreq_free(auth);
+        reply_list = grad_client_decrypt_pairlist(grad_avl_dup(auth->avlist),
+						  auth->authenticator, auth->secret);
+
+        p = (radtest_variable_t*)grad_sym_lookup(vartab, "REPLY");
+        p->type = rtv_avl;
+        p->datum.avl = reply_list;
+        grad_request_free(auth);
 }
 
 /* FIXME: duplicated in radiusd/files.c */
 int
-comp_op(int op, int result)
+comp_op(enum grad_operator op, int result)
 {
         switch (op) {
         default:
-        case OPERATOR_EQUAL:
+        case grad_operator_equal:
                 if (result != 0)
                         return -1;
                 break;
 
-        case OPERATOR_NOT_EQUAL:
+        case grad_operator_not_equal:
                 if (result == 0)
                         return -1;
                 break;
 
-        case OPERATOR_LESS_THAN:
+        case grad_operator_less_than:
                 if (result >= 0)
                         return -1;
                 break;
 
-        case OPERATOR_GREATER_THAN:
+        case grad_operator_greater_than:
                 if (result <= 0)
                         return -1;
                 break;
                     
-        case OPERATOR_LESS_EQUAL:
+        case grad_operator_less_equal:
                 if (result > 0)
                         return -1;
                 break;
                         
-        case OPERATOR_GREATER_EQUAL:
+        case grad_operator_greater_equal:
                 if (result < 0)
                         return -1;
                 break;
@@ -573,33 +566,44 @@ comp_op(int op, int result)
 }
 
 int
-compare_lists(VALUE_PAIR *reply, VALUE_PAIR *sample)
+compare_lists(grad_avp_t *reply_list, grad_avp_t *sample)
 {
         int result = 0;
+	grad_avp_t *reply = grad_avl_dup(reply_list);
         
         for (; sample && result == 0; sample = sample->next) {
-                VALUE_PAIR *p;
+                grad_avp_t *p, *prev = NULL;
 
                 if (sample->attribute > 255)
                         continue;
                 for (p = reply; p && p->attribute != sample->attribute;
-                     p = p->next)
+                     prev = p, p = p->next)
                         ;
                 if (!p)
                         return -1;
                 switch (p->type) {
-                case TYPE_STRING:
+                case GRAD_TYPE_STRING:
                         result = strcmp(sample->avp_strvalue, p->avp_strvalue);
                         break;
-                case TYPE_INTEGER:
-                case TYPE_IPADDR:
+			
+                case GRAD_TYPE_INTEGER:
+                case GRAD_TYPE_IPADDR:
                         result = sample->avp_lvalue - p->avp_lvalue;
                         break;
+			
                 default:
                         result = -1;
                 }
                 result = comp_op(sample->operator, result);
+		if (result == 0) {
+			if (!prev)
+				reply = p->next;
+			else
+				prev->next = p->next;
+			grad_avp_free(p);
+		}
         }
+	grad_avl_free(reply);
         return result;
 }
 
