@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -30,11 +30,9 @@
 #include <ctype.h>
 #include <netinet/in.h>
 
-#include <sysdep.h>
-#include <radutmp.h>
-#include <radius.h>
-#include <radargp.h>
-#include <radpaths.h>
+#include <common.h>
+#include <radius/radutmp.h>
+#include <radius/radargp.h>
 
 void local_who();
 void radius_who();
@@ -96,7 +94,7 @@ static struct {
 	{ "gnu",
 	  "User: (login)(newline)\
 In real life: (gecos)(newline)\
-Logged in: (time)(newline)\
+Logged in: (time :format %c)(newline)\
 NAS: (nas-address)(newline)\
 Port: (nas-port)(newline)\
 CLID: (clid)(newline)\
@@ -230,9 +228,8 @@ main(int  argc, char **argv)
         char *p, *q;
         int index;
 
-        app_setup();
-        initlog(argv[0]);
-        if (rad_argp_parse(&argp, &argc, &argv, 0, &index, NULL))
+        grad_app_setup();
+        if (grad_argp_parse(&argp, &argc, &argv, 0, &index, NULL))
                 return 1;
 
         if (!fmtspec)
@@ -241,7 +238,7 @@ main(int  argc, char **argv)
         if (!fmtspec)
                 fmtspec = lookup_format("default");
 
-	form = radutent_compile_form(fmtspec);
+	form = grad_utent_compile_form(fmtspec);
 	if (!form)
 		exit(1);
 
@@ -249,16 +246,16 @@ main(int  argc, char **argv)
                 filename = radutmp_path;
         
         /* Read the dictionary files */
-        dict_init();
+        grad_dict_init();
         /* Read the "naslist" file. */
-        path = mkfilename(radius_dir, RADIUS_NASLIST);
-	if (nas_read_file(path))
+        path = grad_mkfilename(radius_dir, RADIUS_NASLIST);
+	if (grad_nas_read_file(path))
                 exit(1);
-        efree(path);
+        grad_free(path);
         /* Read realms */
-        path = mkfilename(radius_dir, RADIUS_REALMS);
-	realm_read_file(path, 0, 0, NULL);
-        efree(path);
+        path = grad_mkfilename(radius_dir, RADIUS_REALMS);
+	grad_read_realms(path, 0, 0, NULL);
+        grad_free(path);
         
         /*
          *      See if we are "fingerd".
@@ -322,8 +319,8 @@ local_who()
         struct radutmp rt;
         
         if ((fp = fopen(UTMP_FILE, "r")) == NULL) {
-                radlog(L_ERR, _("can't open file: %s"),
-                       UTMP_FILE);
+                grad_log(L_ERR, _("can't open file: %s"),
+                         UTMP_FILE);
                 return;
         }
 
@@ -349,7 +346,7 @@ local_who()
 #endif
                         tty_to_port(&rt, ut.ut_line);
                         if (want_rad_record(&rt)) 
-                                radutent_print(form, &rt, 1);
+                                grad_utent_print(form, &rt, 1);
                 }
         }
         fclose(fp);
@@ -370,8 +367,10 @@ radius_who()
                 return ;
 
         while (up = rut_getent(file)) {
-                if (want_rad_record(up)) 
-                        radutent_print(form, up, 1);
+		if (up->type == P_ACCT_DISABLED) 
+			printf(_("System accounting is disabled\n"));
+		else if (want_rad_record(up)) 
+                        grad_utent_print(form, up, 1);
         }
         rut_endent(file);
 }
@@ -380,7 +379,7 @@ void
 print_header()
 {
         if (display_header) {
-                printutmp_header(form);
+                grad_utent_print_header(form);
                 display_header = 0;
         }
 }
@@ -388,7 +387,8 @@ print_header()
 int
 want_rad_record(struct radutmp *rt)
 {
-        if (username && strcmp(rt->login, username))
+        if ((username && strcmp(rt->login, username))
+	    || rt->type == P_ACCT_ENABLED) 
                 return 0;
         
         switch (showall) {
